/*
 * blend.cpp
 *
 *  Created on: Jan 22, 2017
 *      Author: joe
 */

#include "blend.h"
#include <random>
#include "simpleplot.h"

blend::blend(std::vector<axis> maxes, float ds, std::vector<float> stimes) {
  if((int)stimes.size() != maxes[2].n) {
    seperr("blend: Please enter a shot time for each shot %d %d.\n",(int)stimes.size(),maxes[2].n);
  }
  init(maxes, ds, stimes, false);
}

blend::blend(std::vector<axis> maxes, float ds, std::vector<float> stimes, bool eshtout) {
  if((int)stimes.size() != maxes[2].n) {
    seperr("blend: Please enter a shot time for each shot %d %d.\n",(int)stimes.size(),maxes[2].n);
  }
  init(maxes, ds, stimes, eshtout);
}

blend::blend(std::vector<axis> maxes, float ds, float maxdev, float mindev) {
  /* Randomize shot times */
  std::vector<float> stimes;
  std::random_device rd; std::mt19937 rng(rd());
  std::uniform_real_distribution<float> uni(maxdev,mindev);
  /* First time is always zero */
  stimes.push_back(0.0);
  for(int is = 1; is < maxes[2].n; ++is) {
    float stime = is*ds;
    auto val = uni(rng);
    stimes.push_back(val + stime);
  }
  init(maxes,ds,stimes,false);
}

blend::blend(std::vector<axis> maxes, float ds, float maxdev, float mindev, bool eshtout) {
  /* Randomize shot times */
  std::vector<float> stimes;
  std::random_device rd; std::mt19937 rng(rd());
  std::uniform_real_distribution<float> uni(maxdev,mindev);
  /* First time is always zero */
  stimes.push_back(0.0);
  for(int is = 1; is < maxes[2].n; ++is) {
    float stime = is*ds;
    auto val = uni(rng);
    stimes.push_back(val + stime);
  }
  init(maxes,ds,stimes,eshtout);
}

void blend::init(std::vector<axis> maxes, float ds, std::vector<float>stimes, bool eshtout) {
  _nt = maxes[0].n; _nrx = maxes[1].n; _ns = maxes[2].n;
  _ds = ds;
  _stimes = stimes;
  _eshtout = eshtout;
  _nbs = static_cast<int>(_stimes.back()/_ds) + 1;
  _ubpb = std::vector<int>(_nbs,0);
  _sp = shotpad(maxes[0]);
  std::vector<axis> taxes, tuaxes, t2axes;
  float dt = maxes[0].d;
  taxes.push_back(axis(_ns*_nt,0.0,dt)); taxes.push_back(maxes[1]);
  tuaxes.push_back(axis(_ns*_nt,0.0,dt)); tuaxes.push_back(maxes[1]);
  tuaxes.push_back(maxes[2]);
  _lsht = new hypercube_float(taxes,true);
  t2axes.push_back(maxes[0]); t2axes.push_back(maxes[1]);
  _osht = new hypercube_float(t2axes,true);
  if(_eshtout) {
    _ubsht = new hypercube_float(taxes,true);
    _ubshts = new hypercube_float(tuaxes,true);
    _esht  = new hypercube_float(t2axes,true);
    _eshts = new hypercube_float(maxes,true);
    /* Build unblended to blended counter */
    for(int is = 0; is < (int)_stimes.size(); ++is) {
      int ibin = static_cast<int>(_stimes[is]/ds + 0.5);
      _ubpb[ibin] += 1;
    }
  }
}

int blend::get_nshots() {
  _run_nshots = true;
  return _nbs;
}

bool blend::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  /* Check if they have run get_nshots */
  if(!_run_nshots) seperr("blend: Please run_nshots to allocate the new number of shots "
      "for your output blended data.\n");

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  _lsht->zero();
  /* Dither shots and put into one super long gather */
  for(int is = 0; is < _ns; ++is) {
    /* Copy shot into temporary array */
    memcpy(_osht->vals,&m->vals[_nt*_nrx*is],sizeof(float)*_nt*_nrx);
    /* Pad shot randomly and add to super gather */
    _sp.set_time(_stimes[is]);
    _sp.forward(true,_osht,_lsht);
    if(_eshtout) {
      /* Get just the encoded shots (no blending) */
      _sp.forward(false,_osht,_ubsht);
      memcpy(&_ubshts->vals[_ns*_nt*_nrx*is],_ubsht->vals,sizeof(float)*_ns*_nt*_nrx);
    }
  }

  /* Cut shots into individual shot gathers */
  int k = 0;
  for(int is = 0; is < _nbs; ++is) {
    _osht->zero();
    copy_shot(_lsht,_osht,is);
    mempcpy(&d->vals[_nt*_nrx*is],_osht->vals,sizeof(float)*_nt*_nrx);
    if(_eshtout) {
      for(int ies = 0; ies < _ubpb[is]; ++ies) {
        memcpy(_ubsht->vals,&_ubshts->vals[_ns*_nt*_nrx*(k)],sizeof(float)*_ns*_nt*_nrx);
        copy_shot(_ubsht,_esht,is);
        memcpy(&_eshts->vals[_nt*_nrx*(k)],_esht->vals,sizeof(float)*_nt*_nrx);
        ++k;
      }
    }
  }
  return true;
}

//TODO: For the adjoint, I need the random shot times generated
//      from the foward
bool blend::adjoint(bool add, my_vector *dat, my_vector *mod) {
  if(!add && mod) mod->zero();

  /* Check if they have run get_nshots */
  if(!_run_nshots) seperr("blend: Please run_nshots to allocate the new number of shots "
      "for your output blended data.\n");

  /* Loop over shot */
  for(int is = 1; is < _nbs; ++is) {
  }
  return true;
}

//TODO: Use the adjoint of the shot pad operator instead
void blend::copy_shot(hypercube_float *sushot, hypercube_float *sishot, int ishot) {
  for(int irx = 0; irx < _nrx; ++irx) {
    for(int it = 0; it < _nt; ++it) {
      sishot->vals[irx*_nt + it] = sushot->vals[irx*_nt*_ns + ishot*_nt + it];
    }
  }
}

void blend::get_encshots(hypercube_float *eshts) {
  if(_eshts != nullptr) {
    if(!_eshts->check_match(eshts)) {
      seperr("The size of the input encoded shots does not match the computed "
          " size of computed encoded shots.\n");
    }
    eshts->set(_eshts->vals);
  }
}
