#include "combo_oper.h"
#include "simpleplot.h"

col_op::col_op(my_operator *op){ 
  ops.push_back(op);
  create_super();

}
col_op::col_op(my_operator *op1,my_operator *op2){ 
  ops.push_back(op1);
  ops.push_back(op2);
  create_super();
  strcpy(name,"COL_OP");
}
/**
 * Constructs a column operator
 * @param op a vector of my_operator pointers
 */
col_op::col_op(std::vector<my_operator *>op){

  /* Save operators to class memeber vector ops */
  for(int i=0; i < (int)op.size(); i++) ops.push_back(op[i]);
  create_super();

}
void col_op::add_op(my_operator *op){

  ops.push_back(op);
  create_super();
}
void col_op::add_ops(std::vector<my_operator *>os){

  for(int i=0; i < (int) os.size(); i++) ops.push_back(os[i]);
  create_super();

}
/**
 * Create the domain and range of the column operator
 * <p>
 * The range is a super vector (multiple outputs) and
 * the domain is a single vector (one model vector)
 */
void col_op::create_super(){
  std::vector < norm_vector*> ds,rs;
  /* Get the range of each operator (could be different for each) */
  for(int i=0; i < (int)ops.size(); i++) rs.push_back(ops[i]->range);
  /* Check that all operators operate on same model vector */
  for(int i=1; i < (int)ops.size(); i++){
    if(!ops[0]->domain->check_match(ops[i]->domain)){
      seperr("ERROR:Domain does not match in array op 0 and %d \n",i);
    }
  }
  if(range!=0) { delete range; delete domain; }

  range=(norm_vector*) new super_vector(rs,true);
  domain=(norm_vector*) ops[0]->domain->clone_space();
}

/**
 * Performs the application of the forward column operator
 * @param add
 * @param model
 * @param data
 * @return
 */
bool col_op::forward(bool add, my_vector *model, my_vector *data) {

  if(!check_domain_range(model,data)){
    seperr("col op match failed(1)\n");
    return false;
  }

  super_vector *d=(super_vector*) data;

  for(int i=0; i < (int)ops.size(); i++){
    if(!ops[i]->forward(add,model,d->vecs[i])){
      fprintf(stderr,"failed running col op %d \n",i);
      assert(1==0);
      return false;
    }
  }
  return true;
}

bool col_op::adjoint(bool add, my_vector *model, my_vector *data){

  if(!check_domain_range(model,data)){
    fprintf(stderr,"col op match failed(2)\n");
    assert(1==0);

    return false;
  }
  super_vector *d=(super_vector*) data;
  bool a=add;
  for(int i=0; i < (int)ops.size(); i++){

    if(!ops[i]->adjoint(a,model,d->vecs[i])){
      fprintf(stderr,"failed running array op %d \n",i);
      assert(1==0);

      return false;
    }

    a=true;
  }
  return true;
}
row_op::row_op(my_operator *op){ 
  ops.push_back(op);
  create_super();

}
row_op::row_op(my_operator *op1,my_operator *op2){ 
  ops.push_back(op1);
  ops.push_back(op2);
  create_super();
  strcpy(name,"ROW_OP");
}

row_op::row_op(std::vector<my_operator *>op){
  for(int i=0; i < (int)op.size(); i++) ops.push_back(op[i]);
  create_super();

}
void row_op::add_op(my_operator *op){

  ops.push_back(op);
  create_super();
}
void row_op::add_ops(std::vector<my_operator *>os){

  for(int i=0; i < (int) os.size(); i++) ops.push_back(os[i]);
  create_super();

}
void row_op::create_super(){
  std::vector < norm_vector*> ds;
  for(int i=0; i < (int)ops.size(); i++){
    ds.push_back(ops[i]->domain);
    if(i!=1 && !ops[i]->range->check_match(ops[0]->range)){
      fprintf(stderr,"Range of two row ops don't match");
      assert(1==2);
    }
  }
  if(range!=0) {
    delete range; delete domain;
  }
  domain=(norm_vector*)new super_vector(ds,true);
  range=(norm_vector*) ops[0]->range->clone_space();

}
bool row_op::forward(bool add, my_vector *model, my_vector *data){
  if(!check_domain_range(model,data)){
    fprintf(stderr,"row op match failed(3)\n");
    return false;
  }
  super_vector *m=(super_vector*) model;
  bool asend=add;
  for(int i=0; i < (int)ops.size(); i++){
    if(!ops[i]->forward(asend,m->vecs[i],data)){
      fprintf(stderr,"failed running row op %d \n",i);
      return false;
    }
    asend=true;
  }
  return true;
}
bool row_op::adjoint(bool add, my_vector *model, my_vector *data){

  if(!check_domain_range(model,data)){
    fprintf(stderr,"row op match failed(3)\n");
    return false;
  }
  super_vector *m=(super_vector*) model;
  for(int i=0; i < (int)ops.size(); i++){

    if(!ops[i]->adjoint(add,m->vecs[i],data)){
      fprintf(stderr,"failed running row op %d \n",i);
      return false;
    }
  }
  return true;
}

diag_op::diag_op(my_operator *op){ 
  ops.push_back(op);
  create_super();

}
diag_op::diag_op(my_operator *op1,my_operator *op2){ 
  ops.push_back(op1);
  ops.push_back(op2);
  create_super();
  strcpy(name,"DIAG_OP");
}
diag_op::diag_op(std::vector<my_operator *>op){
  for(int i=0; i < (int)op.size(); i++) ops.push_back(op[i]);
  create_super();

}
void diag_op::add_op(my_operator *op){

  ops.push_back(op);
  create_super();
}
void diag_op::add_ops(std::vector<my_operator *>os){

  for(int i=0; i < (int) os.size(); i++) ops.push_back(os[i]);
  create_super();

}
void diag_op::create_super(){
  std::vector < norm_vector*> ds,rs;
  for(int i=0; i < (int)ops.size(); i++){
    ds.push_back(ops[i]->domain);
    rs.push_back(ops[i]->range);
  }
  if(range!=0) {
    delete range; delete domain;
  }
  domain=(norm_vector*)new super_vector(ds,true);
  range=(norm_vector*) new super_vector(rs,true);

}
bool diag_op::forward(bool add, my_vector *model, my_vector *data){
  if(!check_domain_range(model,data)){
    fprintf(stderr,"diag op match failed(3)\n");
    return false;
  }
  super_vector *m=(super_vector*) model;
  super_vector *d=(super_vector*) data;
  for(int i=0; i < (int)ops.size(); i++){


    if(!ops[i]->forward(add,m->vecs[i],d->vecs[i])){
      seperr("failed running diagonal op %d \n",i);
      return false;
    }
  }
  return true;
}
bool diag_op::adjoint(bool add, my_vector *model, my_vector *data){

  if(!check_domain_range(model,data)){
    fprintf(stderr,"diag op match failed(3)\n");
    return false;
  }
  super_vector *m=(super_vector*) model;
  super_vector *d=(super_vector*) data;
  for(int i=0; i < (int)ops.size(); i++){
    if(!ops[i]->adjoint(add,m->vecs[i],d->vecs[i])){
      seperr("failed running diagonal op %d \n",i);
      return false;
    }
  }
  return true;
}

bool chain_op::forward(bool add, my_vector *model, my_vector *data){
  if(!check_domain_range(model,data)){
    fprintf(stderr,"chain op match failed\n");
    return false;
  }
  my_vector *d,*r,*dom=model,*ran;
  bool a;

  for(int i=0; i < (int) ops.size(); i++){
    if(i!=(int)ops.size()-1) {
      a=false;
      r=(norm_vector*)ops[i]->range->clone_zero();
      ran=r;
    }
    else {
      a=add;
      ran=data;
    }
    if(!ops[i]->forward(a,dom,ran)){
      fprintf(stderr,"failed running chain op %d \n",i);
      return false;
    }
    if(i!=0) delete dom;
    dom=ran;
  }

  return true;
}

bool chain_op::adjoint(bool add, my_vector *model, my_vector *data){

  if(!check_domain_range(model,data)){
    fprintf(stderr,"array op match failed\n");
    return false;
  }
  my_vector *dom,*d,*ran=data;;
  bool a=false;
  for(int i=(int)ops.size()-1; i >=0; i--){

    if(i!=0) {
      a=false;
      d=(norm_vector*)ops[i]->domain->clone_zero();
      dom=d;
    }
    else {
      a=add;
      dom=model;
    }
    bool x=ops[i]->adjoint(a,dom,ran);
    if(!x){
      fprintf(stderr,"failed running chain op %d \n",i);
      return false;
    }
    if(i!=(int)ops.size()-1) delete ran;
    ran=dom;

  }

  return true;
}
void chain_op::chain_init(std::vector<my_operator*> os){

  if(!check_valid_chain()) {
    fprintf(stderr,"trouble creating chain op \n");
    return;
  }  
  for(int i=0; i< (int)os.size(); i++) ops.push_back(os[i]);

  domain=(norm_vector*)os[0]->domain->clone_space();
  range=(norm_vector*)os[os.size()-1]->range->clone_space();

}
bool chain_op::check_valid_chain(){

  for(int i=0; i < (int) ops.size()-1; i++){
    if(!ops[i]->range->check_match(ops[i+1]->domain)){
      fprintf(stderr,"domain and range don't match when creating super operator (%d %d) \n",i,i+1);
      return false;
    }
  }
  return true;
}

void chain_op::add_op(my_operator *op){
  ops.push_back(op);
  if(!check_valid_chain())
    fprintf(stderr,"trouble adding operator\n");
}

stack_op::stack_op(std::vector<my_operator*> ops){
  _ops.push_back(ops[0]);
  /* Check the domains of each of the operators */
  std::vector<norm_vector*> domains; domains.push_back(_ops[0]->domain);
  /* Get the range of the first operator */
  range=(norm_vector*)_ops[0]->range->clone_space();

  for(int i=1; i < (int)ops.size(); i++){
    _ops.push_back(ops[i]);
    if(!_ops[i]->domain->check_match(domains[0]) || !_ops[i]->range->check_match(range)) {
      fprintf(stderr,"domain and range must be the same for stack op\n");
      assert(1==2);
    }
  }
  domain = new super_vector(domains,true);
  _sc=1;
}

bool stack_op::forward(bool add, my_vector *model, my_vector *data){
  if(!check_domain_range(model,data)){
    fprintf(stderr,"stack op match failed(3)\n");
    return false;
  }
  super_vector* models = (super_vector*) model;
  super_vector* ranges = (super_vector*) range;
  for(int i=0; i < (int)_ops.size();i++) {
    _ops[i]->forward(false,models->vecs[i],ranges->vecs[i]);
  }
  float sc0=0;
  if(add) sc0=1;
  data->scale_add(sc0,ranges->vecs[0],_sc);
  for(int i = 1; i < (int)ranges->vecs.size(); i++) data->scale_add(1.,ranges->vecs[i],_sc);
  return true;
}

bool stack_op::adjoint(bool add, my_vector *model, my_vector *data){
  if(!check_domain_range(model,data)){
    fprintf(stderr,"row op match failed(3)\n");
    return false;
  }
  super_vector* models=(super_vector*) model;
  super_vector *ranges=(super_vector*) range;
  float sc0=0;
  if(add) sc0=1;
  for(int i=0; i < (int)ranges->vecs.size(); i++)  ranges->vecs[i]->scale_add(0.,data,_sc);
  for(int i=0; i < (int)models->vecs.size();i++) _ops[i]->adjoint(add,models->vecs[i],ranges->vecs[i]);
  return true;
}
