/*
 * config_parser.cpp
 *
 *  Created on: Aug 6, 2017
 *      Author: joe
 */

#include <iostream>
#include <regex>
#include <fstream>
#include "error_handle.h"
#include "config_parser.h"
extern "C"{
#include "seplib.h"
}

ConfigParser::ConfigParser(int argc, char **argv) {
  _argc = argc; _argv = argv;
  /* Loop over all arguments splitting the '=' */
  for(int iarg = 1; iarg < argc; ++iarg) {
    std::vector<std::string> keyval = split(argv[iarg],"=");
    if(keyval.size() != 2) continue;
    _ndict["cmd"][keyval[0]] = keyval[1];
  }
  if(_ndict["cmd"].find("par") != _ndict["cmd"].end()) {
    std::string fname = _ndict["cmd"]["par"];
    std::ifstream cfile(fname);
    std::string line, secname;
    while(!cfile.eof()) {
      std::getline(cfile,line);
      if(!line.empty()) {
        if(line.at(0) == '[') {
          secname = split(split(line,"[")[1],"]")[0];
          continue;
        }
        std::vector<std::string> keyval = split(line,"=");
        _ndict[secname][keyval[0]] = keyval[1];
      }
    }
  }
}


ConfigParser::ConfigParser(int argc, char **argv, bool initsep) {
  if(initsep) (initpar(argc,argv));
  _argc = argc; _argv = argv;
  /* Loop over all arguments splitting the '=' */
  for(int iarg = 1; iarg < argc; ++iarg) {
    std::vector<std::string> keyval = split(argv[iarg],"=");
    if(keyval.size() != 2) continue;
    _ndict["cmd"][keyval[0]] = keyval[1];
  }
  if(_ndict["cmd"].find("par") != _ndict["cmd"].end()) {
    std::string fname = _ndict["cmd"]["par"];
    std::ifstream cfile(fname);
    std::string line, secname;
    while(!cfile.eof()) {
      std::getline(cfile,line);
      if(!line.empty()) {
        if(line.at(0) == '[') {
          secname = split(split(line,"[")[1],"]")[0];
          continue;
        }
        std::vector<std::string> keyval = split(line,"=");
        _ndict[secname][keyval[0]] = keyval[1];
      }
    }
  }
}
int ConfigParser::get_int(const std::string &sec, const std::string &key) {
  //TODO: Check if _ndict is empty
  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      throw_invalid_arg("Value '" + key + "' not found in section '" + sec + "'\n");
    }
    else {
      return std::stoi(_ndict[sec][key]);
    }
  }
}

int ConfigParser::get_int(const std::string &sec, const std::string &key, const int &def) {
  //TODO: Check if _ndict is empty
  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      return def;
    }
    else {
      return std::stoi(_ndict[sec][key]);
    }
  }
}

std::vector<int> ConfigParser::get_ints(const std::string &sec, const std::string &key) {

  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      throw_invalid_arg("Value '" + key + "' not found in section '" + sec + "'\n");
    }
    else {
      // Split the comma-delimited list
      std::vector<std::string> svals = split(_ndict[sec][key],",");
      size_t nvals = svals.size();
      std::vector<int> ivals(nvals,0);
      for(size_t ival = 0; ival < nvals; ++ival) {
        ivals[ival] = std::stoi(svals[ival]);
      }
      return ivals;
    }
  }
}

bool ConfigParser::get_bool(const std::string &sec, const std::string &key) {
  //TODO: Check if _ndict is empty
  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      throw_invalid_arg("Value '" + key + "' not found in section '" + sec + "'\n");
    }
    else {
      if(!_ndict[sec][key].compare("y")) {
        return true;
      } else if(!_ndict[sec][key].compare("n")) {
        return false;
      } else {
        throw_invalid_arg("Unrecognized option '" + _ndict[sec][key] +"' for bool");
      }
    }
  }
}

bool ConfigParser::get_bool(const std::string &sec, const std::string &key, const bool &def) {
  //TODO: Check if _ndict is empty
  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      return def;
    }
    else {
      if(!_ndict[sec][key].compare("y")) {
        return true;
      } else if(!_ndict[sec][key].compare("n")) {
        return false;
      } else {
        throw_invalid_arg("Unrecognized option '" + _ndict[sec][key] +"' for bool");
      }
    }
  }
}

float ConfigParser::get_float(const std::string &sec, const std::string &key) {

  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      throw_invalid_arg("Value '" + key + "' not found in section '" + sec + "'\n");
    }
    else {
      return std::stof(_ndict[sec][key]);
    }
  }
}

float ConfigParser::get_float(const std::string &sec, const std::string &key, const float &def) {

  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      return def;
    }
    else {
      return std::stof(_ndict[sec][key]);
    }
  }
}

std::vector<float> ConfigParser::get_floats(const std::string &sec, const std::string &key) {

  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      throw_invalid_arg("Value '" + key + "' not found in section '" + sec + "'\n");
    }
    else {
      // Split the comma-delimited list
      std::vector<std::string> svals = split(_ndict[sec][key],",");
      size_t nvals = svals.size();
      std::vector<float> fvals(nvals,0.0);
      for(size_t ival = 0; ival < nvals; ++ival) {
        fvals[ival] = std::stof(svals[ival]);
      }
      return fvals;
    }
  }
}


double ConfigParser::get_double(const std::string &sec, const std::string &key) {

  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      throw_invalid_arg("Value '" + key + "' not found in section '" + sec + "'\n");
    }
    else {
      return std::stod(_ndict[sec][key]);
    }
  }
}

std::vector<double> ConfigParser::get_doubles(const std::string &sec, const std::string &key) {

  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      throw_invalid_arg("Value '" + key + "' not found in section '" + sec + "'\n");
    }
    else {
      // Split the comma-delimited list
      std::vector<std::string> svals = split(_ndict[sec][key],",");
      size_t nvals = svals.size();
      std::vector<double> dvals(nvals,0.0);
      for(size_t ival = 0; ival < nvals; ++ival) {
        dvals[ival] = std::stod(svals[ival]);
      }
      return dvals;
    }
  }
}

std::string ConfigParser::get(const std::string &sec, const std::string &key) {

  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      throw_invalid_arg("Value '" + key + "' not found in section '" + sec + "'\n");
    }
    else {
      return _ndict[sec][key];
    }
  }
}

std::string ConfigParser::get(const std::string &sec, const std::string &key, const std::string &def) {

  if(_ndict.find(sec) == _ndict.end()) {
    throw_invalid_arg("Section '" + sec + "' not found in config file\n");
  } else {
    if(_ndict[sec].find(key) == _ndict[sec].end()) {
      return def;
    }
    else {
      return _ndict[sec][key];
    }
  }
}

std::vector<std::string> ConfigParser::split(std::string str, std::string delim) {
  std::vector<std::string> splitstr;
  size_t pos = 0;
  std::string token;
  while ((pos = str.find(delim)) != std::string::npos) {
    token = str.substr(0,pos);
    splitstr.push_back(token);
    str.erase(0,pos + delim.length());
  }
  splitstr.push_back(str);
  return splitstr;
}
