/**
 * Conjugate gradient solver for solving
 * the normal equations (Least squares)
 * @author: Joseph Jennings
 * @version: 2017.03.25
 */
#ifndef CONJ_GRAD_H_
#define CONJ_GRAD_H_

#include "my_operator.h"
#include "my_vector.h"
#include "cgstep.h"
#include <vector>
#include <map>

class conj_grad {
  public:
    conj_grad(){_res = 0; _grd = 0; _drs = 0;};
    conj_grad(int niter);
    conj_grad(int niter, bool verb);
    conj_grad(int niter, std::map<std::string, std::string> fouts);
    conj_grad(double gtol, double rtol);
    conj_grad(double rtol);
    conj_grad(int niter, double rtol);
    conj_grad(int niter, double rtol, double gtol);
    void run(my_operator *op, my_vector *dat, my_vector *mod);
    void run(my_operator *op, my_operator* grdtap, my_vector *dat, my_vector *mod);
    void run(my_operator *op, my_operator *rop, my_vector* dat, my_vector* rdat, my_vector* mod, float eps);
    void set_resout(std::string tag);
    void set_mrsout(std::string tag);
    void set_grdout(std::string tag);
    void set_grd1out(std::string tag);
    void set_grd2out(std::string tag);
    void set_modout(std::string tag);
    void set_mod1out(std::string tag);
    void set_mod2out(std::string tag);
    void set_rdnout(std::string tag);
    void set_rmnout(std::string tag);
    void set_rtnout(std::string tag);
    void set_gnout(std::string tag);
    void set_ofout(std::string tag);
    void set_verb(bool verb) { _verb = verb; };
    double get_obj(my_vector *res);
    ~conj_grad(){}

private:
  int _niter = 0;
  double _rtol = 0.0;
  double _gtol = 0.0;
  double _obj = 0.0;
  bool _verb = true;
  bool _alloced = false;
  cgstep _step;
  void output(my_vector *res, my_vector *grad, my_vector *mod, double obj);
  void alloc_solve(my_vector *dat, my_vector *mod);
  void alloc_solvetp(my_vector *dat, my_vector *mod);
  void dealloc_solve();
  void dealloc_solvetp();
  my_vector *_grd, *_grdtp;
  norm_vector *_res, *_drs;
  std::map<std::string,std::string> _fouts;
  float *_one = nullptr;
  int _counter = 0;
};

#endif /* CONJ_GRAD_H_ */
