/*
 * covariance.cpp
 *
 *  Created on: Feb 4, 2017
 *      Author: joe
 */

extern "C" {
#include "seplib.h"
}
#include "covariance.h"

covariance::covariance(const axis at, const int nsamp) {
  _nt = at.n; _dt = at.d;
  if(nsamp > _nt) {
    seperr("covariance: time window (%f) cannot be larger than "
        "total time in signal (%f).\n",nsamp*_dt,_nt*_dt);
  }
  _nsamp = nsamp;
}

float covariance::variance(const hypercube_float *sig) const {
  float var = 0;
  for(int it = 0; it < _nsamp; ++it) {
    var += sig->vals[it]*sig->vals[it];
  }
  var /= _nsamp;
  return var;
}

std::vector<float> covariance::covmatrix(hypercube_float *z, hypercube_float *x) {
  if(!z->check_match(x)) {
    seperr("covmatrix: input signals must be of same length and sampling.\n");
  }
  float covzz = static_cast<float>(z->dot(z)/_nsamp);
  float covzx = static_cast<float>(z->dot(x)/_nsamp);
  float covxz = static_cast<float>(x->dot(z)/_nsamp);
  float covxx = static_cast<float>(x->dot(x)/_nsamp);
  _covmat = { covzz, covzx, covxz, covxx};
  return _covmat;
}
