/**
 * A generic class for computing variance
 * and covariance matrices of multiple time series.
 * Specifically designed for seismic traces
 * @author: Joseph Jennings
 * @version: 2017.02.4
 */
#ifndef COVARIANCE_H
#define COVARIANCE_H_

#include <vector>
#include "hypercube_float.h"
#include "axis.h"

class covariance {
  public:
    covariance();
    covariance(const axis at, const int nsamp);
    std::vector<float> covmatrix(const hypercube_float *z, const hypercube_float *x, const hypercube_float *y);
    std::vector<float> covmatrix(hypercube_float *z, hypercube_float *x);
    float variance(const hypercube_float *sig) const;
    ~covariance(){};
  private:
    int _nt, _nsamp;
    float _dt;
    std::vector<float> _covmat;
};

#endif /* COVARIANCE_H_ */
