/**
 * C++ exception overrides
 * @author: Joseph Jennings
 * @version: 2017.08.19
 */

#pragma once

#include <stdio.h>
#include <string>
#include <stdexcept>
#include <iostream>
#include <sstream>

class Exception: public std::runtime_error {
  public:
    Exception(const std::string &arg, const char *file, const char *func, int line) :
      std::runtime_error(arg) {
      std::ostringstream o;
      o << " File: " << file << " - " << "Function: " << func << " - " << "Line no: " << line << " - \n" << arg;
      msg = o.str();
    }
    std::string msg;
    ~Exception() throw() {}
    const char *what() const throw() {
      return msg.c_str();
    }
  };

class InvalidArg: public std::invalid_argument{
  public:
    InvalidArg(const std::string &arg, const char *file, const char *func, int line) :
      std::invalid_argument(arg) {
      std::ostringstream o;
      o << " File: " << file << " - " << "Function: " << func << " - " << "Line no: " << line << "\n" << arg;
      msg = o.str();
    }
    std::string msg;
    ~InvalidArg() throw() {}
    const char *what() const throw() {
      return msg.c_str();
    }
  };

#define throw_exception(arg) throw Exception(arg, __FILE__, __func__, __LINE__);
#define throw_invalid_arg(arg) throw InvalidArg(arg, __FILE__, __func__, __LINE__);
#define report_line() std::cerr << "Exception caught in main at line: " << __LINE__ << "\n";

