/*
 * igrad2.cpp
 *
 *  Created on: Jan 27, 2017
 *      Author: joe
 */

#include "igrad2.h"
#include "hypercube_float.h"

igrad2::igrad2(std::vector<axis> uaxes) {
  if((int)uaxes.size() != 3)
    seperr("igrad2: Input wavefield must be two dimensional.\n");
  _nz = uaxes[0].n; _idz = 1/uaxes[0].d;
  _nx = uaxes[1].n; _idx = 1/uaxes[1].d;
  _nt = uaxes[2].n;
}

bool igrad2::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(d->get_axis(4).n != 2)
    seperr("igrad2: data n4 must equal 2 in order to hold each component.\n");

  for(int it = 0; it < _nt; ++it) {
    for(int ix = 0; ix < _nx-1; ++ix) {
      for(int iz = 0; iz < _nz-1; ++iz) {
        float val = m->vals[it*_nz*_nx + ix*_nz + iz];
        /* Derivative in z */
        d->vals[0*_nz*_nx*_nt + it*_nz*_nx + ix*_nz + iz] +=
            (m->vals[it*_nz*_nx + ix*_nz + (iz+1)] - val)*_idz;
        /* Derivative in x */
        d->vals[1*_nz*_nx*_nt + it*_nz*_nx + ix*_nz + iz] +=
            (m->vals[it*_nz*_nx + (ix+1)*_nz + iz] - val)*_idx;
      }
    }
  }

  return true;
}

bool igrad2::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(d->get_axis(4).n != 2)
    seperr("igrad2: data n4 must equal 2 in order to hold each component.\n");

  for(int it = 0; it < _nt; ++it) {
    for(int ix = 0; ix < _nx-1; ++ix) {
      for(int iz = 0; iz < _nz-1; ++iz) {
        /* Adjoint z derivative */
        float zval = d->vals[0*_nz*_nx*_nt + it*_nz*_nx + ix*_nz + iz];
        m->vals[it*_nz*_nx + (ix  )*_nz + (iz+1)] += zval;
        m->vals[it*_nz*_nx + (ix  )*_nz + (iz  )] -= zval;
        /* Adjoint x derivative */
        float xval = d->vals[1*_nz*_nx*_nt + it*_nz*_nx + ix*_nz + iz];
        m->vals[it*_nz*_nx + (ix+1)*_nz + (iz  )] += xval;
        m->vals[it*_nz*_nx + (ix  )*_nz + (iz  )] -= xval;
      }
    }
  }

  return true;
}
