/*
 * igrad2cent10th.cpp
 *
 *  Created on: Apr 25, 2017
 *      Author: joe
 */

#include "igrad2cent10th.h"
#include "hypercube_float.h"

igrad2cent10th::igrad2cent10th(std::vector<axis> uaxes) {
  if((int)uaxes.size() != 3)
    seperr("igrad2cent10th: Input wavefield must be two dimensional.\n");
  _nz = uaxes[0].n; _idz = 1/uaxes[0].d;
  _nx = uaxes[1].n; _idx = 1/uaxes[1].d;
  _nt = uaxes[2].n;
}

bool igrad2cent10th::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(d->get_axis(4).n != 2)
    seperr("igrad2: data n4 must equal 2 in order to hold each component.\n");

  int zloc = 0*_nz*_nx*_nt;
  int xloc = 1*_nz*_nx*_nt;
  for(int it = 0; it < _nt; ++it) {
    int off = it*_nz*_nx;
    for(int ix = 5; ix < _nx-5; ++ix) {
      for(int iz = 5; iz < _nz-5; ++iz) {
        /* Derivative in z */
        d->vals[zloc + off + ix*_nz + iz] +=
            _coefs[0]*(m->vals[off + ix*_nz + (iz+1)] - m->vals[off + ix*_nz + (iz-1)])*_idz +
            _coefs[1]*(m->vals[off + ix*_nz + (iz+2)] - m->vals[off + ix*_nz + (iz-2)])*_idz +
            _coefs[2]*(m->vals[off + ix*_nz + (iz+3)] - m->vals[off + ix*_nz + (iz-3)])*_idz +
            _coefs[3]*(m->vals[off + ix*_nz + (iz+4)] - m->vals[off + ix*_nz + (iz-4)])*_idz +
            _coefs[4]*(m->vals[off + ix*_nz + (iz+5)] - m->vals[off + ix*_nz + (iz-5)])*_idz;

        /* Derivative in x */
        d->vals[xloc + off + ix*_nz + iz] +=
            _coefs[0]*(m->vals[off + (ix+1)*_nz + iz] - m->vals[off + (ix-1)*_nz + iz])*_idx +
            _coefs[1]*(m->vals[off + (ix+2)*_nz + iz] - m->vals[off + (ix-2)*_nz + iz])*_idx +
            _coefs[2]*(m->vals[off + (ix+3)*_nz + iz] - m->vals[off + (ix-3)*_nz + iz])*_idx +
            _coefs[3]*(m->vals[off + (ix+4)*_nz + iz] - m->vals[off + (ix-4)*_nz + iz])*_idx +
            _coefs[4]*(m->vals[off + (ix+5)*_nz + iz] - m->vals[off + (ix-5)*_nz + iz])*_idx;
      }
    }
  }

  return true;
}

bool igrad2cent10th::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(d->get_axis(4).n != 2)
    seperr("igrad2: data n4 must equal 2 in order to hold each component.\n");

  int zloc = 0*_nz*_nx*_nt;
  int xloc = 1*_nz*_nx*_nt;
  for(int it = 0; it < _nt; ++it) {
    int off = it*_nz*_nx;
    for(int ix = 5; ix < _nx-5; ++ix) {
      for(int iz = 5; iz < _nz-5; ++iz) {
        m->vals[off + ix*_nz + iz] +=
            /* Z derivative */
            -(_coefs[0]*(d->vals[zloc + off + ix*_nz + (iz+1)] - d->vals[zloc + off + ix*_nz + (iz-1)])*_idz +
            _coefs[1]*(d->vals[zloc + off + ix*_nz + (iz+2)] - d->vals[zloc + off + ix*_nz + (iz-2)])*_idz +
            _coefs[2]*(d->vals[zloc + off + ix*_nz + (iz+3)] - d->vals[zloc + off + ix*_nz + (iz-3)])*_idz +
            _coefs[3]*(d->vals[zloc + off + ix*_nz + (iz+4)] - d->vals[zloc + off + ix*_nz + (iz-4)])*_idz +
            _coefs[4]*(d->vals[zloc + off + ix*_nz + (iz+5)] - d->vals[zloc + off + ix*_nz + (iz-5)])*_idz)
            /* X derivative */
            -(_coefs[0]*(d->vals[xloc + off + (ix+1)*_nz + iz] - d->vals[xloc + off + (ix-1)*_nz + iz])*_idx +
            _coefs[1]*(d->vals[xloc + off + (ix+2)*_nz + iz] - d->vals[xloc + off + (ix-2)*_nz + iz])*_idx +
            _coefs[2]*(d->vals[xloc + off + (ix+3)*_nz + iz] - d->vals[xloc + off + (ix-3)*_nz + iz])*_idx +
            _coefs[3]*(d->vals[xloc + off + (ix+4)*_nz + iz] - d->vals[xloc + off + (ix-4)*_nz + iz])*_idx +
            _coefs[4]*(d->vals[xloc + off + (ix+5)*_nz + iz] - d->vals[xloc + off + (ix-5)*_nz + iz])*_idx);
      }
    }
  }

  return true;
}
