/*
 * integratewfld.cpp
 *
 *  Created on: Feb 14, 2017
 *      Author: joe
 */

#include "integratewfld.h"

integratewfld::integratewfld(std::vector<axis> waxes, float rho) {
  _nz = waxes[0].n; _nx = waxes[1].n; _nt = waxes[2].n;
  _onestp = _nz*_nx;
  std::vector<axis> azx;
  azx.push_back(waxes[0]); azx.push_back(waxes[1]);
  _tmp  = new hypercube_float(azx,true);
  _mslc = new hypercube_float(azx,true);
  _dslc = new hypercube_float(azx,true);
  _irho = 1.0/rho;
}

bool integratewfld::forward(bool add, my_vector *mod, my_vector *dat) {
  if(add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(!m->check_match(d)) {
    seperr("integratewfld: model and data must have same axes\n");
  }

  if(m->get_axis(1).n != _nz && m->get_axis(2).n != _nx && m->get_axis(3).n != _nt) {
    seperr("integratewfld: model and data axes are inconsistent with "
        "axes passed to the constructor.\n");
  }

  /* Initial condition */
  memcpy(d->vals,m->vals,sizeof(float)*_onestp);
  /* Integrate in time */
  for(int it = 1; it < _nt; ++it) {
    memcpy(_tmp->vals,&d->vals[(it-1)*_onestp],sizeof(float)*_onestp);
    memcpy(_mslc->vals,&m->vals[it*_onestp],sizeof(float)*_onestp);
    _dslc->scale_add(1.0,_tmp,1.0,_mslc);
    memcpy(&d->vals[it*_onestp],_dslc->vals,sizeof(float)*_onestp);
  }
  /* Divide by density */
  d->scale(_irho);

  return true;
}

bool integratewfld::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(add && mod) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(!m->check_match(d)) {
    seperr("integratewfld: model and data must have same axes\n");
  }

  if(m->get_axis(1).n != _nz && m->get_axis(2).n != _nx && m->get_axis(3).n != _nt) {
    seperr("integratewfld: model and data axes are inconsistent with "
        "axes passed to the constructor.\n");
  }

  /* Terminal condition */
  memcpy(&m->vals[(_nt-1)*_onestp],&d->vals[(_nt-1)*_onestp],sizeof(float)*_onestp);
  /* Integrate back in time */
  for(int it = _nt-2; it >= 0; --it) {
    memcpy(_tmp->vals,&m->vals[(it+1)*_onestp],sizeof(float)*_onestp);
    memcpy(_dslc->vals,&d->vals[it*_onestp],sizeof(float)*_onestp);
    _mslc->scale_add(1.0,_tmp,1.0,_dslc);
    memcpy(&m->vals[it*_onestp],_mslc->vals,sizeof(float)*_onestp);
  }
  /* Divide by density */
  m->scale(_irho);

  return true;
}
