/**
 * Integrates a wavefield in time.
 * Used for computing the particle velocity
 * from the pressure gradient assuming
 * constant density.
 * @author: Joseph Jennings
 * @version: 2017.02.14
 */
#ifndef INTEGRATEWFLD_H_
#define INTEGRATEWFLD_H_

#include <vector>
#include "my_operator.h"
#include "axis.h"
#include "hypercube_float.h"

class integratewfld : public my_operator {
  public:
    integratewfld(){};
    integratewfld(std::vector<axis> waxes, float rho);
    bool forward(bool add, my_vector *mod, my_vector *dat);
    bool adjoint(bool add, my_vector *mod, my_vector *dat);
    virtual ~integratewfld(){
      delete _tmp; delete _mslc; delete _dslc;
    };
  private:
    int _nx, _nz, _nt, _onestp;
    hypercube_float *_tmp, *_mslc, *_dslc;
    float _irho;
};

#endif /* INTEGRATEWFLD_H_ */
