/**
 * Fast linear interpolation in one
 * dimension
 * @author: Joseph Jennings
 * @version: 2017.12.21
 */
#ifndef LINT_FAST_H_
#define LINT_FAST_H_

#include <vector>
#include "my_operator.h"
#include "hypercube_float.h"
#include "block.h"

class lint_fast: public my_operator {
  public:
    lint_fast(){};
    lint_fast(int nd, int j, int nlag);
    lint_fast(int nd, int j, std::vector<int> nlag);
    bool forward(bool add, my_vector *mod, my_vector *dat);
    bool adjoint(bool add, my_vector *mod, my_vector *dat);
    void print_blockinfo();
    void print_laginfo();
    std::vector<int> get_numfilt();
    ~lint_fast(){
      delete _b0; delete _d0;
    };
  private:
    void find_optimal_sizes(int nd, int j, std::vector<int> &b, std::vector<int> &e);
    int _nd, _nf, _nlag;
    std::vector<int> _lag;
    std::vector<block1d> _blocks;
    hypercube_float *_b0, *_d0;
};

#endif /* LINT_FAST_H_ */
