/**
 * Linearly-varying convolution
 * in one dimension, adjoint
 * returns the model.
 * @author: Joseph Jennings
 * @version: 2017.12.21
 */
#ifndef LV_CONV1DM_H_
#define LV_CONV1DM_H_

#include <vector>
#include "my_operator.h"
#include "hypercube_float.h"
#include "block.h"

class lv_conv1dm : public my_operator {
  public:
    lv_conv1dm(){};
    lv_conv1dm(int nd, int j, std::vector<int> lag, hypercube_float *aux);
    lv_conv1dm(int nd, int j, int nlag, hypercube_float *aux);
    bool forward(bool add, my_vector *mod, my_vector *dat);
    bool adjoint(bool add, my_vector *mod, my_vector *dat);
    void print_blockinfo();
    void print_laginfo();
    void set_zerolag(hypercube_float *flt);
    void get_imp_resp(int i, hypercube_float *iresp);
    std::vector<int> get_numfilt();
    ~lv_conv1dm(){
      delete _b0; delete _d0;
    };
  private:
    void find_optimal_sizes(int nd, int j, int nf, std::vector<int> &b, std::vector<int> &e);
    int _nd, _nf, _nlag;
    std::vector<int> _lag;
    std::vector<block1d> _blocks;
    hypercube_float *_b0, *_d0, *_aux;
};

#endif /* LV_CONV1DM_H_ */
