/*
 * lv_conv1dmask.cpp
 *
 *  Created on: Dec 21, 2017
 *      Author: joe
 */

#include "lv_conv1dmask.h"
#include "hypercube_float.h"

bool lv_conv1dmask::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  if(!dat->check_match(mod)) {
    seperr("lv_conv1dmask: model and data are not the same size.\n");
  }

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nlag = m->get_axis(1).n;
  int nf   = m->get_axis(2).n;

  d->scale_add(1.0,m,1.0);
  /* Set the zero lag to zero */
  for(int if1 = 0; if1 < nf; ++if1) {
    d->vals[if1*nlag] = 0.0;
  }

  return true;
}

bool lv_conv1dmask::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  if(!dat->check_match(mod)) {
    seperr("lv_conv1dmask: model and data are not the same size.\n");
  }

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nlag = d->get_axis(1).n;
  int nf   = d->get_axis(2).n;

  m->scale_add(1.0,d,1.0);
  /* Set the zero lag to zero */
  for(int if1 = 0; if1 < nf; ++if1) {
    m->vals[if1*nlag] = 0.0;
  }

  return true;
}
