/**
 * Linearly-varying convolution in one
 * dimensions for multicomponent data.
 * Adjoint returns the model
 * @author: Joseph Jennings
 * @version: 2018.02.11
 */
#ifndef LV_CONV1DMMC_H_
#define LV_CONV1DMMC_H_

#include <vector>
#include "my_operator.h"
#include "hypercube_float.h"
#include "block.h"

class lv_conv1dmmc : public my_operator {
  public:
      lv_conv1dmmc(){};
      lv_conv1dmmc(int &nd, int &j, std::vector<std::pair<int,int>> &lag, hypercube_float *aux);
      lv_conv1dmmc(int &nd, int &j, std::vector<int> nlag, hypercube_float *aux);
      bool forward(bool add, my_vector *mod, my_vector *dat);
      bool adjoint(bool add, my_vector *mod, my_vector *dat);
      void print_blockinfo();
      void print_laginfo();
      void set_zerolag(hypercube_float *flt);
      void get_imp_resp(int i, hypercube_float *iresp);
      std::vector<int> get_numfilt();
      ~lv_conv1dmmc(){
        delete _b00; delete _d01;
      }
  private:
      void find_optimal_sizes(int nd, int j, int nf, std::vector<int> &b, std::vector<int> &e);
      int _nlag, _nf, _nd, _maxlag2;
      std::vector<std::pair<int,int>> _lag;
      std::vector<block1d> _blocks;
      hypercube_float *_b00, *_d01, *_aux;
};

#endif /* LV_CONV1DMMC_H_ */
