 /**
  * Linearly-varying convolution in two dimensions.
  * Adjoint is the model (filter is the operator)
  * @author: Joseph Jennings
  * @version: 2017.03.23
  */

#ifndef LV_CONV2DM_H_
#define LV_CONV2DM_H_

#include <vector>
#include "my_operator.h"
#include "hypercube_float.h"
#include "block.h"

class lv_conv2dm : public my_operator {
  public:
      lv_conv2dm(){};
      lv_conv2dm(std::vector<int> nd, std::vector<int> &j,
          std::vector<std::pair<int,int>> &lag, hypercube_float *aux);
      lv_conv2dm(std::vector<int> nd, std::vector<int> &j, std::vector<int> nlag, hypercube_float *aux);
      bool forward(bool add, my_vector *mod, my_vector *dat);
      bool adjoint(bool add, my_vector *mod, my_vector *dat);
      void print_blockinfo();
      void print_laginfo();
      void set_zerolag(hypercube_float *flt);
      void set_data(hypercube_float *data);
      void get_imp_resp(int i, int j, hypercube_float *iresp);
      std::vector<int> get_numfilt();
      ~lv_conv2dm(){
        delete _b00; delete _b01; delete _bin;
        delete _d01; delete _d02; delete _d12;
      }
  private:
      void find_optimal_sizes(int nd, int j, int nf, std::vector<int> &b, std::vector<int> &e);
      int _nlag;
      std::vector<int> _nf, _nd;
      std::vector<std::pair<int,int>> _lag;
      std::vector<block2d> _blocks;
      hypercube_float *_b00, *_bin, *_b01, *_d01, *_d02, *_d12, *_aux;
};

#endif /* LV_CONV2DM_H_ */
