/**
 * Linearly-varying convolution in three
 * dimensions. Adjoint returns the filter
 * @author: Joseph Jennings
 * @version: 2017.03.17
 */

#ifndef LV_CONV3DF_H_
#define LV_CONV3DF_H_


#include <vector>
#include "my_operator.h"
#include "hypercube_float.h"
#include "block.h"
#include "triple.h"

class lv_conv3df : public my_operator {
  public:
    lv_conv3df(){};
    lv_conv3df(std::vector<int> nd, std::vector<int> &j,
        std::vector<triple> &lag, hypercube_float *aux);
    lv_conv3df(std::vector<int> nd, std::vector<int> &j, std::vector<int> nlag, hypercube_float *aux);
    bool forward(bool add, my_vector *mod, my_vector *dat);
    bool adjoint(bool add, my_vector *mod, my_vector *dat);
    void print_blockinfo();
    void print_laginfo();
    void set_zerolag(hypercube_float *flt);
    void set_data(hypercube_float *data);
    void get_imp_resp(int i, int j, hypercube_float *iresp);
    std::vector<axis> get_filtaxes();
    ~lv_conv3df(){
      delete _b000; delete _b100; delete _b110; delete _b010;
      delete _d001; delete _d101; delete _d111; delete _d011;
      delete _d00;  delete _bin; delete _d01;
      delete _b00;  delete _b10;
    }
  private:
    void find_optimal_sizes(int nd, int j, int nf, std::vector<int> &b, std::vector<int> &e);
    int _nlag;
    std::vector<int> _nf, _nd;
    std::vector<triple> _lag;
    std::vector<block3d> _blocks;
    hypercube_float *_b000, *_b100, *_b110, *_b010, *_b00, *_b10, *_d001, *_d101, *_d111, *_d011, *_d00, *_bin, *_d01, *_d0, *_aux;
};

#endif /* LV_CONV3DF_H_ */
