/**
 * Linearly-varying convolution in three
 * dimensions. Adjoint returns the filter
 * @author: Joseph Jennings
 * @version: 2017.03.17
 */

#ifndef LV_CONV3DF_H_
#define LV_CONV3DF_H_


#include <vector>
#include "my_operator.h"
#include "hypercube_float.h"
#include "block.h"
#include "triple.h"

class lv_conv3dffast : public my_operator {
  public:
    lv_conv3dffast(){};
    lv_conv3dffast(std::vector<int> nd, std::vector<int> &j,
        std::vector<triple> &lag, hypercube_float *aux);
    lv_conv3dffast(std::vector<int> nd, std::vector<int> &j, std::vector<int> nlag, hypercube_float *aux);
    bool forward(bool add, my_vector *mod, my_vector *dat);
    bool adjoint(bool add, my_vector *mod, my_vector *dat);
    void print_blockinfo();
    void print_laginfo();
    void set_zerolag(hypercube_float *flt);
    void set_data(hypercube_float *data);
    void get_imp_resp(int i, int j, hypercube_float *iresp);
    std::vector<axis> get_filtaxes();
    ~lv_conv3dffast(){
      delete _lag1; delete _lag2; delete _lag3;
    }
  private:
    void find_optimal_sizes(int nd, int j, int nf, std::vector<int> &b, std::vector<int> &e);
    int _nlag;
    std::vector<int> _nf, _nd;
    std::vector<triple> _lag;
    int *_lag1, *_lag2, *_lag3;
    std::vector<block3d> _blocks;
    hypercube_float *_aux;
};

#endif /* LV_CONV3DF_H_ */
