/*
 * lv_conv3dmask.cpp
 *
 *  Created on: Mar 23, 2017
 *      Author: joe
 */

#include "lv_conv3dmask.h"
#include "hypercube_float.h"

bool lv_conv3dmask::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  if(!dat->check_match(mod)) {
    seperr("lv_conv2dmask: model and data are not the same size.\n");
  }

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nlag = m->get_axis(1).n;
  int nf1  = m->get_axis(2).n;
  int nf2  = m->get_axis(3).n;
  int nf3  = m->get_axis(4).n;

  d->scale_add(1.0,m,1.0);
  /* Set the zero lag to zero */
  for(int if3 = 0; if3 < nf3; ++if3) {
    for(int if2 = 0; if2 < nf2; ++if2) {
      for(int if1 = 0; if1 < nf1; ++if1) {
        d->vals[if3*nf2*nf1*nlag + if2*nf1*nlag + if1*nlag] = 0.0;
      }
    }
  }

  return true;
}

bool lv_conv3dmask::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  if(!dat->check_match(mod)) {
    seperr("lv_conv2dmask: model and data are not the same size.\n");
  }

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nlag = d->get_axis(1).n;
  int nf1  = d->get_axis(2).n;
  int nf2  = d->get_axis(3).n;
  int nf3  = d->get_axis(4).n;

  m->scale_add(1.0,d,1.0);
  /* Set the zero lag to zero */
  for(int if3 = 0; if3 < nf3; ++if3) {
    for(int if2 = 0; if2 < nf2; ++if2) {
      for(int if1 = 0; if1 < nf1; ++if1) {
        m->vals[if3*nf2*nf1*nlag + if2*nf1*nlag + if1*nlag] = 0.0;
      }
    }
  }

  return true;
}
