/*
 * mctcai.cpp
 *
 *  Created on: May 3, 2017
 *      Author: joe
 */

#include "mctcai.h"

mctcai::mctcai(std::vector<my_operator*> ops) {
  if((int)ops.size() != 4) seperr("mctcai assumes a maximum of four operators.\n");
  _ops = ops;
}

void mctcai::create_super() {
  /* Ranges of operators in same row should match */
  if(!_ops[0]->range->check_match(_ops[1]->range) || !_ops[2]->range->check_match(_ops[3]->range)) {
    seperr("Range of operators for a given row do not match\n");
  }
  /* Domains of operators in same column should match */
  if(!_ops[0]->domain->check_match(_ops[2]->domain) || !_ops[1]->domain->check_match(_ops[3]->domain)) {
    seperr("Domain of operators for a given column do not match\n");
  }

  /* Build domain and range */
  std::vector<norm_vector*> ds, rs;
  for(int i = 0; i < (int)_ops.size(); ++i) {
    ds.push_back(_ops[i]->domain);
    rs.push_back(_ops[i]->range);
  }
  if(range!=0) {
    delete range; delete domain;
  }
  domain=(norm_vector*)new super_vector(ds,true);
  range=(norm_vector*) new super_vector(rs,true);
}

bool mctcai::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  super_vector *m = (super_vector*)mod;
  super_vector *d = (super_vector*)dat;

  /* First row */
  _ops[0]->forward(add ,m->vecs[0],d->vecs[0]);
  _ops[1]->forward(true,m->vecs[1],d->vecs[0]);

  /* Second row */
  _ops[2]->forward(add , m->vecs[0],d->vecs[1]);
  _ops[3]->forward(true, m->vecs[1],d->vecs[1]);

  return true;
}

bool mctcai::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  super_vector *m = (super_vector*)mod;
  super_vector *d = (super_vector*)dat;

  /* First row */
  _ops[0]->adjoint(add ,m->vecs[0],d->vecs[0]);
  _ops[2]->adjoint(true,m->vecs[0],d->vecs[1]);

  /* Second row */
  _ops[1]->adjoint(add ,m->vecs[1],d->vecs[0]);
  _ops[3]->adjoint(true,m->vecs[1],d->vecs[1]);

  return true;
}
