/*
 * padvel.cpp
 *
 *  Created on: Jan 28, 2017
 *      Author: joe
 */

#include "padvel.h"
#include "hypercube_float.h"
#include "simpleplot.h"

padvel::padvel(int zloc, int xloc, int ord) {
  _zloc = zloc; _xloc = xloc; _shift = ord/2;
}

bool padvel::forward(bool add, my_vector *mod, my_vector *dat) {
  dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nzp = d->get_axis(1).n; int nxp = d->get_axis(2).n;
  int nz  = m->get_axis(1).n; int nx  = m->get_axis(2).n;

  /* Check axes */
  if(nzp < nz || nxp < nx) {
    seperr("Output padded image will be smaller than input. Please try again with nzp > nz.\n");
  }
  if(_xloc >= nx || _zloc >= nz) {
    seperr("Initial location is out of bounds. Please try again with xloc < nx or zloc < nz.\n");
  }

  /* First pad with zeros */
  for(int ix = 0; ix < nx; ++ix) {
    for(int iz = 0; iz < nz; ++iz) {
      d->vals[(ix+_xloc)*nzp + iz + _zloc] = m->vals[ix*nz + iz];
    }
  }

  /* Extend z = 0 edge */
  for(int ix = _shift; ix < nxp-_shift; ++ix) {
    for(int iz = _shift; iz < _zloc; ++iz) {
      float ext = d->vals[ix*nzp + _zloc];
      d->vals[ix*nzp + iz] = ext;
    }
  }

  /* Extend x = 0 edge */
  for(int ix = _shift; ix < _xloc; ++ix) {
    for(int iz = _shift; iz < nzp-_shift; ++iz) {
      float ext = d->vals[(_xloc)*nzp + iz];
      d->vals[ix*nzp + iz] = ext;
    }
  }

  /* Extend z = nz edge */
  for(int ix = _shift; ix < nxp-_shift; ++ix) {
    for(int iz = nz+_zloc; iz < nzp-_shift; ++iz) {
      float ext = d->vals[ix*nzp + nz+_zloc-1];
      d->vals[ix*nzp + iz] = ext;
    }
  }

  /* Extend x = nx edge */
  for(int ix = nx+_xloc; ix < nxp-_shift; ++ix) {
    for(int iz = _shift; iz < nzp-_shift; ++iz) {
      float ext = d->vals[(nx+_xloc-1)*nzp + iz];
      d->vals[ix*nzp + iz] = ext;
    }
  }

  return true;
}

bool padvel::adjoint(bool add, my_vector *mod, my_vector *dat) {
  mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nzp = d->get_axis(1).n; int nxp = d->get_axis(2).n;
  int nz  = m->get_axis(1).n; int nx  = m->get_axis(2).n;

  /* Check axes */
  if(nzp < nz || nxp < nx) {
    seperr("Output padded image will be smaller than input. Please try again with nzp > nz.\n");
  }
  if(_xloc >= nx || _zloc >= nz) {
    seperr("Initial location is out of bounds. Please try again with xloc < nx or zloc < nz.\n");
  }

  for(int ix = 0; ix < nx; ++ix) {
    for(int iz = 0; iz < nz; ++iz) {
      m->vals[ix*nz + iz] = d->vals[(ix+_xloc)*nzp + iz+_zloc];
    }
  }

  return true;
}
