/*
 * polydiv.cpp
 *
 *  Created on: Mar 16, 2017
 *      Author: joe
 */

#include "polydiv.h"

polydiv::polydiv(hypercube_float *flt) {
  _flt = flt;
  _nf = flt->get_axis(1).n;
}

bool polydiv::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  if(!mod->check_match(dat)) {
    seperr("polydivfor: data and model should be same size\n");
  }

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nm = m->get_axis(1).n;

  for(int id = 0; id < nm; ++id) {
    float tt = m->vals[id];
    for(int ic = 0; ic < std::min(_nf, id); ++ic) {
      int im = id - 1 - ic;
      tt -= _flt->vals[ic] * d->vals[im];
    }
    d->vals[id] += tt;
  }
  return true;
}

bool polydiv::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  if(!mod->check_match(dat)) {
    seperr("polydivadj: data and model should be same size\n");
  }

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  int nm = m->get_axis(1).n;

  for(int im = nm-1; im >= 0; --im) {
    float tt = d->vals[im];
    for(int ic = 0; ic < std::min(_nf,(nm-1)-im); ++ic) {
      int id = im + 1 + ic;
      tt -= _flt->vals[ic] * m->vals[id];
    }
    m->vals[im] += tt;
  }
  return true;
}
