/*
 * sctvpef.cpp
 *
 *  Created on: Jun 14, 2017
 *      Author: joe
 */

#include <math.h>
#include "seplib.h"
#include "sctvpef.h"

sctvpef::sctvpef(int na, int gap, float lambda) {
  _na = na; _gap = gap; _lambda = lambda; _verb = false;
  if(_lambda != 0.0) {
    _eps = 1.0/_lambda;
  } else {
    seperr("Please enter a non-zero value for lambda.\n");
  }
  _A = std::vector<float>(_na,0.0); _A[0] = 1.0;
}

sctvpef::sctvpef(int na, int gap, float lambda, bool verb) {
  _na = na; _gap = gap; _lambda = lambda; _verb = verb;
  if(_lambda != 0.0) {
    _eps = 1.0/_lambda;
  } else {
    seperr("Please enter a non-zero value for lambda.\n");
  }
  _A = std::vector<float>(na,0); _A[0] = 1.0;
}

sctvpef::sctvpef(std::vector<float> A, int gap, float lambda, bool verb) {
  _A = A; _na = A.size(); _gap = gap; _lambda = lambda; _verb = verb;
  if(_lambda != 0.0) {
    _eps = 1.0/_lambda;
  } else {
    seperr("Please enter a non-zero value for lambda.\n");
  }
  _A[0] = 1.0;
}

sctvpef::sctvpef(std::vector<float> A, int gap, float lambda) {
  _A = A; _na = A.size(); _gap = gap; _lambda = lambda; _verb = false;
  if(_lambda != 0.0) {
    _eps = 1.0/_lambda;
  } else {
    seperr("Please enter a non-zero value for lambda.\n");
  }
  _A[0] = 1.0;
}

std::vector<float> sctvpef::findpef(hypercube_float *dat, hypercube_float *rsd) {
  /* Check the data are residual are same size */
  if(!dat->get_axis(1).same_axis(rsd->get_axis(1))) {
    seperr("Input data and output residual must have same axis.\n");
  }
  int nt = dat->get_axis(1).n;
  /* Initialize scaling terms (sigmay and simgae) */
  float sumsq = 0.0;
  for(int it = 0; it < nt; ++it) {
    sumsq += dat->vals[it]*dat->vals[it];
  }
  float sigy = sqrt(sumsq/(float)_lambda);
  float sige = sigy/2;

  if(_verb) {
    fprintf(stderr,"sigy=%f sige=%f\n",sigy,sige);
    fprintf(stderr,"lambda=%f epsilon=%f\n",_lambda,_eps);
  }
  /* Loop over all predictable samples */
  for(int it = _na; it < nt; ++it) {
    rsd->vals[it] = 0.0;
    /* Perform forward convolution */
    for(int itau = 0; itau < _na; ++itau) {
      rsd->vals[it] += _A[itau]*dat->vals[it-itau];
    }
    /* Compute running variance */
    sige = (1.0-_eps) * sige + _eps * sqrt(rsd->vals[it]*rsd->vals[it]);
    sigy = (1.0-_eps) * sigy + _eps * sqrt(dat->vals[it]*dat->vals[it]);
    /* Perform adjoint convolution (correlation) */
    for(int itau = _gap; itau < _na; ++itau) {
      _A[itau] -= _eps/(sige*sigy) * rsd->vals[it] * dat->vals[it-itau];
    }
  }
  return _A;
}
