/*
 * shotpad.cpp
 *
 *  Created on: Jan 22, 2017
 *      Author: joe
 */

#include "shotpad.h"
#include "hypercube_float.h"

shotpad::shotpad(axis taxis){
  _nt = taxis.n; _ot = taxis.o; _dt = taxis.d;
  _ntshift = 0;
}

shotpad::shotpad(axis taxis, float time) {
  _nt = taxis.n; _ot = taxis.o; _dt = taxis.d;
  if(time < 0) seperr("shotpad: input time cannot be < 0.\n");
  _ntshift = static_cast<int>(time/_dt);
}

void shotpad::set_time(float time) {
  if(time < 0) seperr("shotpad: input time cannot be < 0.\n");
  _ntshift = static_cast<int>(time/_dt);
}

bool shotpad::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(!m->get_axis(1).same_axis(axis(_nt,_ot,_dt)))
    seperr("shotpad: model axis must be same as axis passed to constructor.\n");

  int nrx = m->get_axis(2).n;
  int ntp = d->get_axis(1).n;

  for(int irx = 0; irx < nrx; ++irx) {
    for(int it = 0; it < _nt; ++it) {
      if(it + _ntshift < ntp) {
        d->vals[irx*ntp + it + _ntshift] += m->vals[irx*_nt + it];
      }
    }
  }
  return true;
}

bool shotpad::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) mod->zero();

  hypercube_float *m = (hypercube_float*)mod;
  hypercube_float *d = (hypercube_float*)dat;

  if(!m->get_axis(1).same_axis(axis(_nt,_ot,_dt)))
    seperr("shotpad: model axis must be same as axis passed to constructor.\n");

  int nrx = m->get_axis(2).n;
  int ntp = d->get_axis(1).n;

  for(int irx = 0; irx < nrx; ++irx) {
    for(int it = 0; it < _nt; ++it) {
      if(it + _ntshift < ntp) {
        m->vals[irx*_nt + it] += d->vals[irx*ntp  + _ntshift + it];
      }
    }
  }
  return true;
}
