extern "C" {
#include "seplib.h"
}
#include <boost/lexical_cast.hpp>
#include <complex>
#include <fftw3.h>
#include "simpleplot.h"
#include "gnuplot-iostream.h"

namespace simpleplot {

  void plot(hypercube_float *data) {

    if(data->get_ndim() == 1) {
      simpleplot::plot1d(data);
    }
    if(data->get_ndim() == 2) {
      simpleplot::plot2d(data);
    }
  }

  void plot1d(hypercube_float *data) {
    /* Create a vector of pairs for GnuPlot */
    std::vector<std::pair<float, float> > tmp1d;
    for(int k = 0; k < data->get_n123(); ++k) {
      tmp1d.push_back(std::make_pair(static_cast<float>(k*data->get_axis(1).d),
          static_cast<float>(data->vals[k])));
    }
    /* Plot with GnuPlot */
    Gnuplot gp;
    gp << get_range(data);
    gp << "plot '-' with lines title 'Simple Plot 1D'\n";
    gp.send1d(tmp1d);
  }

  void plot1d2(hypercube_float *data1, hypercube_float *data2, bool clip1) {
     /* Check arguments */
     if(!data1->check_match(data2)) {
       seperr("For overlay, signals must have same axes.\n");
     }
     /* Create vector of pairs */
     std::vector<std::pair<float, float> > tmp1d1, tmp1d2;
     for(int k = 0; k < data1->get_n123(); ++k) {
       tmp1d1.push_back(std::make_pair(static_cast<float>(k*data1->get_axis(1).d),
           static_cast<float>(data1->vals[k])));
       tmp1d2.push_back(std::make_pair(static_cast<float>(k*data2->get_axis(1).d),
           static_cast<float>(data2->vals[k])));
     }
     /* Plot with GnuPlot */
     Gnuplot gp;
     std::string range;
     if(clip1) {
       range = get_range(data1);
     } else {
       range = get_range(data2);
     }
     gp << range;
     gp << "plot '-' with lines title 'data1', '-' with lines title 'data2'\n";
     gp.send1d(tmp1d1);
     gp.send1d(tmp1d2);
   }

  void plot1d(my_vector *data) {
    hypercube_float* d = (hypercube_float*)data;
    /* Create a vector of pairs for GnuPlot */
    std::vector<std::pair<float, float> > tmp1d;
    for(int k = 0; k < d->get_n123(); ++k) {
      tmp1d.push_back(std::make_pair(static_cast<float>(k*d->get_axis(1).d),
          static_cast<float>(d->vals[k])));
    }
    /* Plot with GnuPlot */
    Gnuplot gp;
    gp << get_range(d);
    gp << "plot '-' with lines title 'Simple Plot 1D'\n";
    gp.send1d(tmp1d);
  }

  void plot1dimp(hypercube_float *data) {
    /* Create a vector of pairs for GnuPlot */
    std::vector<std::pair<float, float> > tmp1d;
    for(int k = 0; k < data->get_n123(); ++k) {
      tmp1d.push_back(std::make_pair(static_cast<float>(k*data->get_axis(1).d),
          static_cast<float>(data->vals[k])));
    }
    /* Plot with GnuPlot */
    Gnuplot gp;
    gp << get_range(data);
    gp << "plot '-' with impulses title 'Stem Plot 1D', '-' with points pt 7\n";
    gp.send1d(tmp1d);
    gp.send1d(tmp1d);
  }

  void plot1dimp2(hypercube_float *data1, hypercube_float *data2, bool clip1) {
    /* Check arguments */
    if(!data1->check_match(data2)) {
      seperr("For impulse overlay, signals must have same axes.\n");
    }
    /* Create vector of pairs */
    std::vector<std::pair<float, float> > tmp1d1, tmp1d2;
    for(int k = 0; k < data1->get_n123(); ++k) {
      tmp1d1.push_back(std::make_pair(static_cast<float>(k*data1->get_axis(1).d),
          static_cast<float>(data1->vals[k])));
      tmp1d2.push_back(std::make_pair(static_cast<float>(k*data2->get_axis(1).d),
          static_cast<float>(data2->vals[k])));
    }
    /* Plot with GnuPlot */
    Gnuplot gp;
    std::string range;
    if(clip1) {
      range = get_range(data1);
    } else {
      range = get_range(data2);
    }
    gp << range;
    gp << "plot '-' with impulses title 'data1', '-' with points pt 7, '-' with impulses title 'data2', '-' with points pt 7\n";
    gp.send1d(tmp1d1);
    gp.send1d(tmp1d1);
    gp.send1d(tmp1d2);
    gp.send1d(tmp1d2);
  }

  void plot1dimp3(hypercube_float *data1, hypercube_float *data2, hypercube_float *data3, int wichclp) {
    /* Check arguments */
    if(!data1->check_match(data2) || !data1->check_match(data3) || !data2->check_match(data3)) {
      seperr("For impulse overlay, signals must have same axes.\n");
    }
    /* Create vector of pairs */
    std::vector<std::pair<float, float> > tmp1d1, tmp1d2, tmp1d3;
    for(int k = 0; k < data1->get_n123(); ++k) {
      tmp1d1.push_back(std::make_pair(static_cast<float>(k*data1->get_axis(1).d),
          static_cast<float>(data1->vals[k])));
      tmp1d2.push_back(std::make_pair(static_cast<float>(k*data2->get_axis(1).d),
          static_cast<float>(data2->vals[k])));
      tmp1d3.push_back(std::make_pair(static_cast<float>(k*data3->get_axis(1).d),
          static_cast<float>(data3->vals[k])));
    }
    Gnuplot gp;
    std::string range;
    if(wichclp == 1) {
      range = get_range(data1);
    } else if(wichclp == 2) {
      range = get_range(data2);
    } else if(wichclp == 3) {
      range = get_range(data3);
    } else {
      seperr("Please enter 1, 2, or 3 for choosing the clip\n.");
    }
    gp << "plot '-' with impulses title 'data1', '-' with points pt 7, '-' with impulses title 'data2', '-' with points pt 7, '-' with impulses title 'data3', '-' with points pt 7\n";
    gp.send1d(tmp1d1);
    gp.send1d(tmp1d1);
    gp.send1d(tmp1d2);
    gp.send1d(tmp1d2);
    gp.send1d(tmp1d3);
    gp.send1d(tmp1d3);
  }

  void plot1d(std::vector<float> data, axis myaxis) {
    /* Create a vector of pairs for GnuPlot */
    std::vector<std::pair<float, float> > tmp1d;
    for(int k = 0; k < myaxis.n; ++k) {
      tmp1d.push_back(std::make_pair(static_cast<float>(k*myaxis.d),
          static_cast<float>(data[k])));
    }
    /* Plot with GnuPlot */
    Gnuplot gp;
    gp << get_range(data, myaxis);
    gp << "plot '-' with lines title 'Simple Plot 1D'\n";
    gp.send1d(tmp1d);
  }

  void plot1dspec(hypercube_float* data) {
    int nt = data->get_axis(1).n;
    int nw = nt/2;
    float dw = 1.0/(static_cast<float>(nt)*data->get_axis(1).d);
    std::vector<axis> aw; aw.push_back(axis(nw,0.0,dw));
    hypercube_float *spec = new hypercube_float(aw,true);
    std::complex<double> *in  = new std::complex<double>[nt];
    std::complex<double> *out = new std::complex<double>[nt];
    /* Compute FFT */
    for(int it = 0; it < nt; ++it) in[it].real(data->vals[it]);
    fftw_plan p = fftw_plan_dft_1d(nt,reinterpret_cast<fftw_complex*>(in),
        reinterpret_cast<fftw_complex*>(out),FFTW_FORWARD,FFTW_ESTIMATE);
    fftw_execute(p);
    /* Amplitude spectrum */
    for(int iw = 0; iw < nw; ++iw) {
      spec->vals[iw] = sqrt(out[iw].real()*out[iw].real() + out[iw].imag()*out[iw].imag());
    }
    plot1d(spec);
    delete spec; delete in; delete out;
  }

  void plot1dimp(std::vector<float> data, axis myaxis) {
    /* Create a vector of pairs for GnuPlot */
    std::vector<std::pair<float, float> > tmp1d;
    for(int k = 0; k < myaxis.n; ++k) {
      tmp1d.push_back(std::make_pair(static_cast<float>(k*myaxis.d),
          static_cast<float>(data[k])));
    }
    /* Plot with GnuPlot */
    Gnuplot gp;
    gp << get_range(data, myaxis);
    gp << "plot '-' with impulses title 'Impulse Plot 1D'\n";
    gp.send1d(tmp1d);
  }

  //TODO: Add argument for colorscale
  void plot2d(hypercube_float *data) {
    int n1 = data->get_axis(1).n;
    int n2 = data->get_axis(2).n;
    std::vector<std::vector<float> > image;
    for (int k1 = 0; k1 < n1; ++k1) {
      std::vector<float> row;
      for (int k2 = 0;  k2 < n2; ++k2) {
        row.push_back(data->vals[k2*n1 + k1]);
      }
      image.push_back(row);
    }
    std::string samp1 = boost::lexical_cast<std::string>(data->get_axis(1).d);
    std::string samp2 = boost::lexical_cast<std::string>(data->get_axis(2).d);
    Gnuplot gp;
    gp << get_range2d("'/home/joe/dev/gnuplot-palettes/moreland.pal'");
    gp << "plot '-' binary" << gp.binFmt2d(image, "array") <<  "with image\n";
    gp.sendBinary2d(image);
  }

  void plot2d(my_vector *dat) {
    hypercube_float* data = (hypercube_float*) dat;
    int n1 = data->get_axis(1).n;
    int n2 = data->get_axis(2).n;
    std::vector<std::vector<float> > image;
    for (int k1 = 0; k1 < n1; ++k1) {
      std::vector<float> row;
      for (int k2 = 0;  k2 < n2; ++k2) {
        row.push_back(data->vals[k2*n1 + k1]);
      }
      image.push_back(row);
    }
    std::string samp1 = boost::lexical_cast<std::string>(data->get_axis(1).d);
    std::string samp2 = boost::lexical_cast<std::string>(data->get_axis(2).d);
    Gnuplot gp;
    gp << get_range2d("'/home/joe/dev/gnuplot-palettes/moreland.pal'");
    gp << "plot '-' binary" << gp.binFmt2d(image, "array") <<  "with image\n";
    gp.sendBinary2d(image);
  }
  void plot2d(norm_vector *dat) {
    hypercube_float* data = (hypercube_float*) dat;
    int n1 = data->get_axis(1).n;
    int n2 = data->get_axis(2).n;
    std::vector<std::vector<float> > image;
    for (int k1 = 0; k1 < n1; ++k1) {
      std::vector<float> row;
      for (int k2 = 0;  k2 < n2; ++k2) {
        row.push_back(data->vals[k2*n1 + k1]);
      }
      image.push_back(row);
    }
    std::string samp1 = boost::lexical_cast<std::string>(data->get_axis(1).d);
    std::string samp2 = boost::lexical_cast<std::string>(data->get_axis(2).d);
    Gnuplot gp;
    gp << get_range2d("'/home/joe/dev/gnuplot-palettes/moreland.pal'");
    gp << "plot '-' binary" << gp.binFmt2d(image, "array") <<  "with image\n";
    gp.sendBinary2d(image);
  }

  void plot2d(std::vector<float> data, std::vector<axis> axes) {
    int n1 = axes[0].n;
    int n2 = axes[1].n;
    std::vector<std::vector<float> > image;
    for (int k1 = 0; k1 < n1; ++k1){
      std::vector<float> row;
      for(int k2 = 0; k2 < n2; ++k2) {
        row.push_back(data[k2*n1 + k1]);
      }
      image.push_back(row);
    }
    std::string samp1 = boost::lexical_cast<std::string>(axes[0].d);
    std::string samp2 = boost::lexical_cast<std::string>(axes[1].d);
    Gnuplot gp;
    gp << get_range2d("'/home/joe/dev/gnuplot-palettes/moreland.pal'");
    gp << "plot '-' binary" << gp.binFmt2d(image, "array") <<  "with image\n";
    gp.sendBinary2d(image);
  }

  //TODO: Add a title for frame number
  void plot_slice2d3(hypercube_float *dat, int frame) {
    std::vector <axis> ax2;
    ax2.push_back(dat->get_axis(1)); ax2.push_back(dat->get_axis(2));
    hypercube_float *tood = new hypercube_float(ax2,true);
    int n1 = dat->get_axis(1).n; int n2 = dat->get_axis(2).n;
    int offset = frame * n1 * n2;

    for(int i2 = 0; i2 < n2; ++i2) {
      for(int i1 = 0; i1 < n1; ++i1) {
        tood->vals[i2*n1 + i1] = dat->vals[offset + i2*n1 + i1];
      }
    }
    plot2d(tood);
    delete tood;
  }

  std::string get_range(hypercube_float *data) {
    float minx = data->get_axis(1).o;
    float maxx = data->get_axis(1).n*data->get_axis(1).d;
    float miny = data->my_min();
    float maxy = data->my_max();
    if (miny == maxy) miny = 0.0;
    std::string sminx = boost::lexical_cast<std::string>(minx);
    std::string smaxx = boost::lexical_cast<std::string>(maxx);
    std::string xrange = "[" + sminx + ":" + smaxx + "]";
    std::string sminy = boost::lexical_cast<std::string>(miny);
    std::string smaxy = boost::lexical_cast<std::string>(maxy);
    std::string yrange = "[" + sminy + ":" + smaxy + "]";
    std::string range = "set xrange " + xrange + "\nset yrange " + yrange + "\n";
    return range;
  }

  std::string get_range(std::vector<float> data, axis myaxis) {
    float minx = myaxis.o;
    float maxx = myaxis.n * myaxis.d;
    float miny = *std::min_element(std::begin(data), std::end(data));
    float maxy = *std::max_element(std::begin(data), std::end(data));
    if(miny == maxy) miny = 0.0;
    std::string sminx = boost::lexical_cast<std::string>(minx);
    std::string smaxx = boost::lexical_cast<std::string>(maxx);
    std::string xrange = "[" + sminx + ":" + smaxx + "]";
    std::string sminy = boost::lexical_cast<std::string>(miny);
    std::string smaxy = boost::lexical_cast<std::string>(maxy);
    std::string yrange = "[" + sminy + ":" + smaxy + "]";
    std::string range = "set xrange " + xrange + "\nset yrange " + yrange + "\n";
    return range;
  }

  std::string get_range2d() {
    return "set yrange [* : *] reverse\n";
  }

  std::string get_range2d(std::string cscale) {
    std::string settings = "filename = " + cscale +
        "\nload filename \nset size ratio 1 \n"
        "set yrange [* : *] reverse\n";

    return settings;
  }

  std::string get_range2d(hypercube_float *data) {
    float minx = data->get_axis(1).o;
    float maxx = data->get_axis(1).n * data->get_axis(1).d;
    float miny = data->get_axis(2).o;
    float maxy = data->get_axis(2).n * data->get_axis(2).d;
    float mina = data->my_min();
    float maxa = data->my_max();
    std::string sminx  = boost::lexical_cast<std::string>(minx);
    std::string smaxx  = boost::lexical_cast<std::string>(maxx);
    std::string xsamp  = boost::lexical_cast<std::string>(data->get_axis(1).d);
    std::string xrange = "[" + sminx + ":" + smaxx + "]";
    std::string sminy  = boost::lexical_cast<std::string>(miny);
    std::string smaxy  = boost::lexical_cast<std::string>(maxy);
    std::string yrange = "[" + sminy + ":" + smaxy + "]";
    std::string smina  = boost::lexical_cast<std::string>(mina);
    std::string smaxa  = boost::lexical_cast<std::string>(maxa);
    std::string arange = "[" + smina + ":" + smaxa + "]";
    std::string range = "set size ratio 1 \nset xrange " + xrange + "\nset yrange " + yrange +
        " reverse \nset cbrange " + arange + "\n";
    return range;
  }
}
