/*
 * weight.cpp
 *
 *  Created on: Nov 10, 2016
 *      Author: joe
 */

#include "weight.h"
#include "simpleplot.h"

weight::weight(hypercube_float *weights) {
  _weights = weights;
  _exp = 1;
}

weight::weight(hypercube_float *weights, float exp) {
  _weights = weights;
  _exp = exp;
}

bool weight::forward(bool add, my_vector *mod, my_vector *dat) {
  if(!add && dat) dat->zero();

  if(!mod->check_match(dat))
    seperr("Error: data and model are not same size\n");

  hypercube_float* m = (hypercube_float*)mod;
  hypercube_float* d = (hypercube_float*)dat;

  if(_weights->get_n123() != m->get_n123() && _weights->get_n123() != d->get_n123())
    seperr("Error: weights is not same size as data and model\n");

  for(int k = 0; k < d->get_n123(); ++k) {
    d->vals[k] += scale * pow(_weights->vals[k],_exp) * m->vals[k];
  }

  return true;
}

bool weight::adjoint(bool add, my_vector *mod, my_vector *dat) {
  if(!add && mod) mod->zero();

  if(!dat->check_match(mod))
    seperr("Error: data and model are not same size\n");

  hypercube_float* m = (hypercube_float*)mod;
  hypercube_float* d = (hypercube_float*)dat;

  if(_weights->get_n123() != m->get_n123() && _weights->get_n123() != d->get_n123())
    seperr("Error: weights is not same size as data and model\n");

  for(int k = 0; k < d->get_n123(); ++k) {
    m->vals[k] += scale * pow(_weights->vals[k],_exp) * d->vals[k];
  }

  return true;
}
