#ifndef WEIGHT_H_
#define WEIGHT_H_
/**
 *  A weight operator (diagonal matrix)
 *  @author: Joseph Jennings, Stanford University
 *  @version: 2016.11.10
 */

#include "my_operator.h"
#include "hypercube_float.h"

class weight : public my_operator {
  public:
    weight(){} //Default constructor
    /**
     * Constructs a diagonal weight operator
     * @param weights the weights to be applied to the
     *        input vector
     */
    weight(hypercube_float* weights);
    /**
     * Constructs a diagonal weight operator
     * raised to some power n
     * @param weights the weights to be applied to the
     *        input vector
     * @param exp the exponent to which the weight operator
     *        will be raised
     */
    weight(hypercube_float* weights, float exp);
    /**
     * Applies the forward of the weight operator
     * @param add flag of whether to add to the previous input
     * @param mod the model Fm=d
     * @param dat the data  d=F*m
     * @return
     */
    bool forward(bool add, my_vector *mod, my_vector *dat);
    /**
     * Applies the the adjoint of the weight operator
     * @param add flag of whether to add to the previous input
     * @param mod the model Fm=d
     * @param dat the data  d=F*m
     * @return
     */
    bool adjoint(bool add, my_vector *mod, my_vector *dat);
    ~weight(){
      delete _weights;
    } //Default destructor

  private:
    hypercube_float *_weights;
    float _exp;
};

#endif /* WEIGHT_H_ */
