import os, sys, ConfigParser
import numpy as np
import subprocess

# Create config parser
config = ConfigParser.RawConfigParser()
config.read(sys.argv[1])

# Shot 1 parameters
sfile1  = config.get( 'shot1', 'datfile' )
linnum1 = sfile1.split('/')[-1].split('All')[-1].split(".H")[0]
snum1   = config.getint( 'shot1', 'snum' )
t0s1    = config.getfloat( 'shot1', 't0' )
t1s1    = config.getfloat( 'shot1', 't1' )
assert(t0s1 > 0.0), "t0 must be greater than 0s"
assert(t1s1 < 10.0), "t1 must be less than 10s"
assert(t1s1 > t0s1), "t1 must be greater than t0"
shift1 = config.getfloat( 'shot1', 'shift' )

# Shot 2 parameters
sfile2  = config.get( 'shot2', 'datfile' )
linnum2 = sfile2.split('/')[-1].split('All')[-1].split(".H")[0]
snum2   = config.getint( 'shot2', 'snum' )
t0s2    = config.getfloat( 'shot2', 't0' )
t1s2    = config.getfloat( 'shot2', 't1' )
assert(t0s2 > 0.0), "t0 must be greater than 0s"
assert(t1s2 < 10.0), "t1 must be less than 10s"
assert(t1s2 > t0s2), "t1 must be greater than t0"
shift2 = config.getfloat( 'shot2', 'shift' )

# Get output prefix
oprefix = config.get( 'blending', 'oprefix' )
opref1 = "L" + str(linnum1) + '-' + str(snum1)
opref2 = "L" + str(linnum2) + '-' + str(snum2)

# Get verbosity flag and output dir
verb = config.getint( 'general', 'verb' )
odir = config.get( 'general', 'odir' )
if(os.path.exists(odir) == False):
  os.mkdir(odir)

# Get component combinations
comps = config.get( 'multicmp' , 'combo' )
compsl = comps.split(',')

# Component dictionary
compf = {}
compf['h'] = 3
compf['v'] = 2
compf['x'] = 1
compf['i'] = 0

# sampling
dt = 0.002

# Window shot 1
s0s1 = int(t0s1/dt); s1s1 = int(t1s1/dt)
ns1 = s1s1 - s0s1
for key, value in compf.iteritems():
  wind1 = "Window n3=1 f3=%d f2=%d j2=4 n1=%d f1=%d < %s > %s/%s-%s.H"%(snum1,value,ns1,s0s1,sfile1,odir,opref1,key)
  if(verb): print wind1
  sp = subprocess.check_call(wind1,shell=True)

# Window shot 2
s0s2 = int(t0s2/dt); s1s2 = int(t1s2/dt)
ns2 = s1s2 - s0s2
for key, value in compf.iteritems():
  wind2 = "Window n3=1 f3=%d f2=%d j2=4 n1=%d f1=%d < %s > %s/%s-%s.H"%(snum2,value,ns2,s0s2,sfile2,odir,opref2,key)
  if(verb): print wind2
  sp = subprocess.check_call(wind2,shell=True)

# Apply shift to shot 1
if(shift1 != 0.0):
  ssamp1 = int(shift1/dt)
  for key, value in compf.iteritems():
    if(ssamp1 < 0):
      ssamp1 = abs(ssamp1)
      pad1 = "Pad end1=%d < %s/%s-%s.H | Window f1=%d > %s/%senc-%s.H"%(ssamp1,odir,opref1,key,ssamp1,odir,opref1,key)
    else:
      pad1 = "Pad beg1=%d < %s/%s-%s.H | Window n1=%d > %s/%senc-%s.H"%(ssamp1,odir,opref1,key,ns1,odir,opref1,key)
    if(verb): print pad1
    sp = subprocess.check_call(pad1,shell=True)

# Apply shift to shot 2
if(shift2 != 0.0):
  ssamp2 = int(shift2/dt)
  for key, value in compf.iteritems():
    if(ssamp2 < 0):
      ssamp2 = abs(ssamp2)
      pad2 = "Pad end1=%d < %s/%s-%s.H | Window f1=%d > %s/%senc-%s.H"%(ssamp2,odir,opref2,key,ssamp2,odir,opref2,key)
    else:
      pad2 = "Pad beg1=%d < %s/%s-%s.H | Window n1=%d > %s/%senc-%s.H"%(ssamp2,odir,opref2,key,ns2,odir,opref2,key)
    if(verb): print pad2
    sp = subprocess.check_call(pad2,shell=True)

# Blend shots
for key, value in compf.iteritems():
  ofile = odir + "/" + oprefix + "-%s.H"%(key)
  if(shift1 != 0.0 and shift2 == 0.0):
    bld = "Add %s/%senc-%s.H %s/%s-%s.H > %s"%(odir,opref1,key,odir,opref2,key,ofile)
  elif(shift1 == 0.0 and shift2 != 0.0):
    bld = "Add %s/%s-%s.H %s/%senc-%s.H > %s"%(odir,opref1,key,odir,opref2,key,ofile)
  elif(shift1 != 0.0 and shift2 != 0.0):
    bld = "Add %s/%senc-%s.H %s/%senc-%s.H > %s"%(odir,opref1,key,odir,opref2,key,ofile)
  else:
    bld = "Add %s/%s-%s.H %s/%s-%s.H > %s"%(odir,opref1,key,odir,opref2,key,ofile)
  if(verb): print bld
  sp = subprocess.check_call(bld,shell=True)

# Append for multicomponent
for comb in compsl:
  # Append shift for shot 1
  if(shift1 != 0.0):
    append= " "
    for comp in comb:
      append += "%s/%senc-%s.H "%(odir,opref1,comp)
    cat1 = "Cat axis=3 %s > %s/%senc-%s.H"%(append,odir,opref1,comb)
    if(verb): print cat1
    sp = subprocess.check_call(cat1,shell=True)
  else:
    append = " "
    for comp in comb:
      append += "%s/%s-%s.H "%(odir,opref1,comp)
    cat1 = "Cat axis=3 %s > %s/%s-%s.H"%(append,odir,opref1,comb)
    if(verb): print cat1
    sp = subprocess.check_call(cat1,shell=True)
  # Append shift for shot 2
  if(shift2 != 0.0):
    append = " "
    for comp in comb:
      append += "%s/%senc-%s.H "%(odir,opref2,comp)
    cat2 = "Cat axis=3 %s > %s/%senc-%s.H"%(append,odir,opref2,comb)
    if(verb): print cat2
    sp = subprocess.check_call(cat2,shell=True)
  else:
    append = " "
    for comp in comb:
      append += "%s/%s-%s.H "%(odir,opref2,comp)
    cat2 = "Cat axis=3 %s > %s/%s-%s.H"%(append,odir,opref2,comb)
    if(verb): print cat2
    sp = subprocess.check_call(cat2,shell=True)
  # Append blended
  append = " "
  for comp in comb:
    append += "%s/%s-%s.H "%(odir,oprefix,comp)
    catb = "Cat axis=3 %s > %s/%s-%s.H"%(append,odir,oprefix,comb)
    if(verb): print catb
  sp = subprocess.check_call(catb,shell=True)

# Remove temporary files

