#################################
# Creates acquisition geometry
# plots for each line of the
# Kietta Med. Sea survey
# @author: Joseph Jennings
# @version: 2017.05.12
#################################

import sys, os, glob
import math
import argparse
import subprocess, commands

def find_line_max(linedir,vrb):
  seps = glob.glob(linedir+"*.H")
  sxmin = -1; sxmax = -1; symin = -1; symax = -1;
  gxmin = -1; gxmax = -1; gymin = -1; gymax = -1;
  for sep in seps:
    tup = commands.getstatusoutput("Attr3dhead < %s"%(sep))
    head = tup[1].split()

    # Get SX
    if head[51] != 'sx':
      print "sx is not in usual spot. Are these the Kietta data? Exiting..."
      sys.exit()
    sxmini = float(head[52])
    sxmaxi = float(head[53])
    if vrb > 1:
      print("%s sxmin=%f sxmax=%f"%(sep,sxmini,sxmaxi))
    # Check if shot gather
    if sxmini != sxmaxi:
      print "sxmin != sxmax. Are these the Kietta data (shot gathers)? Exiting..."
    # Get min and max
    if sxmin == -1:
      sxmin = sxmini;
    elif sxmin > sxmini:
      sxmin = sxmini
    if sxmax == -1:
      sxmax = sxmaxi;
    elif sxmax < sxmaxi:
      sxmax = sxmaxi

    # Get SY
    if head[58] != 'sy':
      print "sy is not in usual spot. Are these the Kietta data? Exiting..."
      sys.exit()
    symini = float(head[59])
    symaxi = float(head[60])
    if vrb > 1:
      print("%s symin=%f symax=%f"%(sep,symini,symaxi))
    # Check if shot gather
    if symini != symaxi:
      print "symin != symax. Are these the Kietta data (shot gathers)? Exiting..."
    # Get min and max
    if symin == -1:
      symin = symini;
    elif symin > symini:
      symin = symini
    if symax == -1:
      symax = symaxi;
    elif symax < symaxi:
      symax = symaxi

    # Get GX
    if head[72] != 'gx':
      print "gx is not in usual spot. Are these the Kietta data? Exiting..."
      sys.exit()
    gxmini = float(head[73])
    gxmaxi = float(head[74])
    if vrb > 1:
      print("%s gxmin=%f gxmax=%f"%(sep,gxmini,gxmaxi))
    if gxmini < 0:
      print "Shot %s has a negative gx coordinate. Continuing..."%(sep)
      gxmini = gxmin
    # Get min and max
    if gxmin == -1:
      gxmin = gxmini;
    elif gxmin > gxmini:
      gxmin = gxmini
    if gxmax == -1:
      gxmax = gxmaxi;
    elif gxmax < gxmaxi:
      gxmax = gxmaxi

    # Get GY
    if head[79] != 'gy':
      print "gy is not in usual spot. Are these the Kietta data? Exiting..."
      sys.exit()
    gymini = float(head[80])
    gymaxi = float(head[81])
    if vrb > 1:
      print("%s gymin=%f gymax=%f"%(sep,gymini,gymaxi))
    if gymini < 0:
      print "Shot %s has a negative gy coordinate. Continuing..."%(sep)
      gymini = gymin
    # Get min and max
    if gymin == -1:
      gymin = gymini;
    elif gymin > gymini:
      gymin = gymini
    if gymax == -1:
      gymax = gymaxi;
    elif gymax < gymaxi:
      gymax = gymaxi

  if vrb > 1:
    print("sxmin=%f sxmax=%f symin=%f symax=%f"%(sxmin,sxmax,symin,symax))
    print("gxmin=%f gxmax=%f gymin=%f gymax=%f"%(gxmin,gxmax,gymin,gymax))

  xmin = 0; xmax = 0; ymin = 0; ymax = 0;
  # Get x min and max
  if(sxmin < gxmin):
    xmin = sxmin
  else:
    xmin = gxmin
  if(sxmax > gxmax):
    xmax = sxmax
  else:
    xmax = gxmax
  # Get y min and max
  if(symin < gymin):
    ymin = symin
  else:
    ymin = gymin
  if(symax > gymax):
    ymax = symax
  else:
    ymax = gymax

  return xmin, xmax, ymin, ymax

# Input arguments
parser = argparse.ArgumentParser()
parser.add_argument("line")
parser.add_argument("-fout",action="store",dest="fout")
parser.add_argument("-mi1",action="store",dest="minx")
parser.add_argument("-mi2",action="store",dest="miny")
parser.add_argument("-ma1",action="store",dest="maxx")
parser.add_argument("-ma2",action="store",dest="maxy")
parser.add_argument("-subx",action="store",dest="subx")
parser.add_argument("-suby",action="store",dest="suby")
parser.add_argument("-vrb",action="store",dest="vrb")
parser.add_argument("-cln",action="store",dest="cln")
parser.add_argument("-snum",action="store",dest="snum")

args = parser.parse_args()

line = args.line

# Get the output movie file
if args.fout == None:
  fout = None
else:
  fout = args.fout

# Get the input shot number
if args.snum == None:
  fsnum = None
else:
  fsnum = int(args.snum)

assert(fout != None or fsnum != None), "Must provide either a shot number or output movie file."

# Verbosity flag
if args.vrb == None:
  vrb = 0
else:
  vrb = int(args.vrb)

# Clean up flag
if args.cln == None:
  cln = 1
else:
  cln = int(args.cln)

# Get minima and maxima for plotting
if args.minx == None:
  minx, maxx, miny, maxy= find_line_max(line,vrb)
  print("minx=%f maxx=%f miny=%f maxy=%f"%(minx,maxx,miny,maxy))
  # Round up and round down
  minx = minx - 500;  maxx = maxx + 500;
  miny = miny - 1000; maxy = maxy + 1000;
else:
  #minx = float(args.minx) - 500; maxx = float(args.maxx) + 500;
  #miny = float(args.miny) - 1000; maxy = float(args.maxy) + 1000;
  minx = float(args.minx); maxx = float(args.maxx);
  miny = float(args.miny); maxy = float(args.maxy);
  print("minx=%f maxx=%f miny=%f maxy=%f"%(minx,maxx,miny,maxy))

# Get headermath subtraction
if args.subx != None or args.suby != None:
  if args.subx != None:
    subx = float(args.subx)
  else:
    subx = 0.0
  if args.suby != None:
    suby = float(args.suby)
  else:
    suby = 0.0

# Get files over which to loop
if(fsnum != None):
  seps = glob.glob(line + "*%s*.H"%(fsnum))
else:
  seps = glob.glob(line+"*.H")

# Make output fig directory
odir = line.split('/')[5]
sp = subprocess.check_call("mkdir -p ./Fig/%s"%(odir),shell=True)

movcmd = "vppen "

d = {}

for sep in seps:
  snum = sep.split('/')[-1].split('_')[1]
  d[snum] = sep
  # First do headermath to subtract axes
  cmd1 = "Headermath key1='sx' eqn1='(sx-%f)/%d' key2='sy' \
      eqn2='(sy-%f)/%d' key3='gx' eqn3='(gx-%f)/%d' key4='gy' \
      eqn4='(gy-%f)/%d' < %s > hmath.H"%(subx,1,suby,1,subx,1,suby,1,sep)
  if vrb >= 1:
    print cmd1
  sp = subprocess.check_call(cmd1,shell=True)
  # Then window the source and receiver coordinate
  cmd2 = "Window n2=1 f2=0 j2=4 n1=1 f1=6 < hmath.H@@ > sx.H"
  if vrb >= 1:
    print cmd2
  sp = subprocess.check_call(cmd2,shell=True)
  cmd3 = "Window n2=1 f2=0 j2=4 n1=1 f1=7 < hmath.H@@ > sy.H"
  if vrb >= 1:
    print cmd3
  sp = subprocess.check_call(cmd3,shell=True)
  cmd4 = "Cmplx sx.H sy.H | Graph out=./Fig/%s/sxy%s.v label1='x (m)' labelfat=2 \
          label2='y (m)' symbol='*' plotfat=3 symbolsz=12 plotcol=5 labelrot=n\
          min1=%f max1=%f min2=%f max2=%f title=' ' > /dev/null"%(odir,snum,minx,maxx,miny,maxy)
  if vrb >= 1:
    print cmd4
  sp = subprocess.check_call(cmd4,shell=True)
  cmd5 = "Window f2=0 j2=4 n1=1 f1=9 < hmath.H@@ > gx.H"
  if vrb >= 1:
    print cmd5
  sp = subprocess.check_call(cmd5,shell=True)
  cmd6 = "Window f2=0 j2=4 n1=1 f1=10 < hmath.H@@ > gy.H"
  if vrb >= 1:
    print cmd6
  sp = subprocess.check_call(cmd6,shell=True)
  cmd7 = "Cmplx gx.H gy.H | Graph out=./Fig/%s/gxy%s.v label1=' ' \
          label2=' ' symbol=o plotfat=2 symbolsz=5 labelrot=n \
          min1=%f max1=%f min2=%f max2=%f title=' ' > /dev/null"%(odir,snum,minx,maxx,miny,maxy)
  if vrb >= 1:
    print cmd7
  sp = subprocess.check_call(cmd7,shell=True)
  cmd8 = "vp_Overlay ./Fig/%s/sxy%s.v ./Fig/%s/gxy%s.v > ./Fig/%s/sgxy%s.v"%(odir,snum,odir,snum,odir,snum)
  if vrb >= 1:
    print cmd8
  sp = subprocess.check_call(cmd8,shell=True)
  if vrb >= 1:
    print " "

if(len(seps) > 1):
  for key in sorted(d.iterkeys()):
    movcmd += "./Fig/%s/sgxy%s.v "%(odir,key)

  movcmd += " > %s"%(fout)
  # Make movie
  print movcmd
  sp = subprocess.check_call(movcmd,shell=True)

# Clean up temp files
if(cln != 0):
  figclean = "rm -f ./Fig/%s/sxy.v ./Fig/%s/gxy.v"%(odir,odir)
  sp = subprocess.check_call(figclean,shell=True)

sepclean = "Rm sx.H sy.H gx.H gy.H"
#sp = subprocess.check_call(sepclean,shell=True)
