import os, sys, ConfigParser
import seppy
import numpy as np
import subprocess

# Create config parser
config = ConfigParser.RawConfigParser()
config.read(sys.argv[1])

# PEF 1 parameters
sfile1   = config.get( 'pef1', 'datfile' )
sname1,_ = os.path.splitext(sfile1)
lag1     = config.get( 'pef1', 'lag'     )
lags1    = map(int,lag1.split(','))
j1       = config.get( 'pef1', 'j'       )
niter1   = config.getint( 'pef1', 'niter'   )

# PEF 2 parameters
sfile2   = config.get( 'pef2', 'datfile' )
sname2,_ = os.path.splitext(sfile2)
lag2     = config.get( 'pef2', 'lag'     )
lags2    = map(int,lag2.split(','))
j2       = config.get( 'pef2', 'j'       )
niter2   = config.getint( 'pef2', 'niter'   )

# Deblending parameters
dbfile   = config.get( 'deblending', 'datfile' )
dbname,_ = os.path.splitext(dbfile)
niterdb  = config.getint( 'deblending', 'niter' )

# General parameters
verb = config.getint( 'general', 'verb' )
idir = config.get( 'general', 'idir' )
assert(os.path.exists(idir) == True), "Input dir does not exist."
odir = config.get( 'general', 'odir' )
if(os.path.exists(odir) == False):
  os.mkdir(odir)

# Find the largest lags

lags = [max(lags1[0],lags2[0]), max(lags1[1],lags2[1]), lags1[2]]

# Find padding parameters
pad1 = 0; pad2=0
pad1b = 0; pad1e = 0
if(lags[0]%2 == 0): 
  pad1 = lags[0]/2
  pad1b = pad1 - 1
  pad1e = pad1
else: 
  pad1 = (lags[0] - 1)/2
  pad1b = pad1e = pad1
pad2 = lags[1] - 1

# Pad shot 1
padcmd1 = "Pad beg1=%d end1=%d end2=%d end3=1 < %s/%s > %s/%s-pad.H"%(pad1b,pad1e,pad2,idir,sfile1,odir,sname1)
if(verb): print padcmd1
sp = subprocess.check_call(padcmd1,shell=True)

# Pad shot 2
padcmd2 = "Pad beg1=%d end1=%d end2=%d end3=1 < %s/%s > %s/%s-pad.H"%(pad1b,pad1e,pad2,idir,sfile2,odir,sname2)
if(verb): print padcmd2
sp = subprocess.check_call(padcmd2,shell=True)

# Pad blended data
padcmdbld = "Pad beg1=%d end1=%d end2=%d end3=1 < %s/%s > %s/%s-pad.H"%(pad1b,pad1e,pad2,idir,dbfile,odir,dbname) 
if(verb): print padcmdbld
sp = subprocess.check_call(padcmdbld,shell=True)

# Estimate PEF 1
pef1cmd = "./Bin/LvPef3d.x nlag=%s j=%s verb=1 \
    rmov=%s/%s-rmov.H niter=%d < %s/%s-pad.H > %s/%s-pef.H"%(lag1,j1,odir,sname1,niter1,odir,sname1,odir,sname1)
if(verb): print pef1cmd
sp = subprocess.check_call(pef1cmd,shell=True)

# Estimate PEF 2
pef2cmd = "./Bin/LvPef3d.x nlag=%s j=%s verb=1 \
    rmov=%s/%s-rmov.H niter=%d < %s/%s-pad.H > %s/%s-pef.H"%(lag2,j2,odir,sname2,niter2,odir,sname2,odir,sname2)
if(verb): print pef2cmd
sp = subprocess.check_call(pef2cmd,shell=True)

# Deblending
dblinv = "./Bin/PefDeblndInv3d.x spef=%s/%s-pef.H npef=%s/%s-pef.H verb=1 \
    niter=%d rmov=%s/%s-rmov.H smmov=%s/%s-smov.H nmmov=%s/%s-nmov.H \
    < %s/%s-pad.H > %s/%s-sig.H \
    noizout=%s/%s-noz.H"%(odir,sname1,odir,sname2,niterdb,odir,dbname,odir,dbname,odir,dbname,odir,dbname,odir,dbname,odir,dbname)
if(verb): print dblinv
sp = subprocess.check_call(dblinv,shell=True)

