import sys, argparse, ConfigParser
import seppy
import numpy as np

# Parse the config file
conf_parser = argparse.ArgumentParser(add_help=False)
conf_parser.add_argument("-c", "--conf_file",
                         help="Specify config file", metavar="FILE")
args, remaining_argv = conf_parser.parse_known_args()
defaults = { 
    "n1" : 400,
    "o1" : 0.0,
    "d1" : 0.004,
    "frq": 1.0,
    "phz": 0.0,
    "amp": 1.0,
    }   
if args.conf_file:
    config = ConfigParser.SafeConfigParser()
    config.read([args.conf_file])
    defaults = dict(config.items("Defaults"))

# Parse the other arguments
# Don't surpress add_help here so it will handle -h
parser = argparse.ArgumentParser(parents=[conf_parser],description=__doc__,
    formatter_class=argparse.ArgumentDefaultsHelpFormatter)

parser.set_defaults(**defaults)
parser.add_argument("-n1",help="n1")
parser.add_argument("-o1",help="o1")
parser.add_argument("-d1",help="d1")
parser.add_argument("-frq",help="frequency")
parser.add_argument("-phz",help="phase")
parser.add_argument("-amp",help="Amplitude")
parser.add_argument("out",help="Output file")
args = parser.parse_args(remaining_argv)

n1 = int(args.n1); o1 = float(args.o1); d1 = float(args.d1)
frq = float(args.frq); phi = float(args.phz); amp = float(args.amp)

w = 2*np.pi*frq

#print "dt=%f w=%f 2/w=%f"%(d1,w,2/w)
if(d1 > 2/w):
  print "Stability condition violated. Exiting"
  sys.exit()

t    = np.zeros(n1)
t[0] = o1; t[1] = d1
pcos = np.zeros(n1)

# Initial conditions
pcos[0] = amp*np.cos(phi)
pcos[1] = pcos[0] - amp * w * d1 * np.sin(phi)
# FD coefficients
a1 = (2 - w**2 * d1**2)
a2 = -1
# Model sinusoid
for it in range(2,n1):
  t[it] = it*d1
  pcos[it] = a1*pcos[it-1] + a2*pcos[it-2]

sep = seppy.sep(sys.argv)
oaxes = seppy.axes([n1],[o1],[d1])
#graphargs = "plotfat=2 title=' ' label1='Time (s)' label2='Amplitude' \
#    labelfat=2 legend=y curvelabel='Predicted:True' legendtitle=' '"
#sep.pltgraph([oaxes,oaxes],[pcos,amp*np.cos(w*t+phi)],
#    graphargs=graphargs,figname='./Fig/pcosep.pdf')

sep.write_file("out",oaxes,pcos)
# Write filter coefficients to file
fp = open(sep.get_fname("out"),"a")
fp.write("a1=%f,a2=%f"%(a1,a2))

