from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt
import numpy as np
import argparse

def column(matrix, i):
  return [row[i] for row in matrix]

# Read in file
parser = argparse.ArgumentParser()
parser.add_argument("fin")
parser.add_argument("dim")
parser.add_argument("-shft",action="store",dest="shft")
parser.add_argument("-fout",action="store",dest="fout")
args = parser.parse_args()
if args.shft == None:
  shft = 0
else:
  shft = int(args.shft)
fin = args.fin
dim = args.dim

f = open(fin)
lines = f.readlines()

lags = []

for line in lines:
  if "lag" in line:
    if dim == "2d" or dim == "1d":
      lags.append([int(line.split(' ')[1].split('=')[1]),int(line.split(' ')[2].split('=')[1])])
    elif dim == "3d":
      lags.append([int(line.split(' ')[1].split('=')[1]),int(line.split(' ')[2].split('=')[1]),int(line.split(' ')[3].split('=')[1])])

fig = plt.figure()
ax = fig.add_subplot(111, projection = "3d")

if dim == "2d":
  maxz = max(column(lags,0))
  minz = min(column(lags,0))
  maxx = max(column(lags,1))
  minx = min(column(lags,1))
  maxy = 0
  miny = 0
elif dim == "3d":
  maxz = max(column(lags,0))
  minz = min(column(lags,0))
  maxx = max(column(lags,1))
  minx = min(column(lags,1))
  maxy = max(column(lags,2))
  miny = min(column(lags,2))
elif dim == "1d":
  maxz = max(column(lags,0))
  minz = min(column(lags,0))
  maxx = 0
  minx = 0
  maxy = 0
  miny = 0


ax.set_xlabel("\nReceiver index",fontsize=18)
ax.set_ylabel("\nComponent",fontsize=18) 
ax.set_zlabel("Time index",fontsize=18)
ax.set_xlim3d(minx,maxx+7)
ax.set_ylim3d(miny,maxy+7) 
ax.set_zlim3d(minz-5,maxz+shft)
ax.tick_params(labelsize=18)

for i in range(len(lags)):
  if dim == "2d":
    if(lags[i][0] == 0 and lags[i][1] == 0):
      #print lags[i][1],0,lags[i][0]
      ax.bar3d(lags[i][1],0,-lags[i][0],1,1,1,color='r')
    else:
      #print lags[i][1],0,lags[i][0]
      ax.bar3d(0,lags[i][1],-lags[i][0],1,1,1,color='w')
  elif dim =="3d":
    if(lags[i][0] == 0 and lags[i][1] == 0 and lags[i][2] == 0):
      ax.bar3d(lags[i][1],lags[i][2],-lags[i][0],1,1,1,color='r')
    else:
      ax.bar3d(lags[i][1],lags[i][2],-lags[i][0],1,1,1,color='w')
  elif dim == "1d":
    if(i > 4): break
    if(lags[i][0] == 0):
      #print lags[i][1],0,lags[i][0]
      ax.bar3d(0,0,-lags[i][0],1,1,1,color='r')
    else:
      #print lags[i][1],0,lags[i][0]
      ax.bar3d(0,0,-lags[i][0],1,1,1,color='w')


plt.gca().invert_zaxis()
ax.view_init(elev=15,azim=-133)
if args.fout != None:
  plt.savefig(args.fout,bbox_inches='tight')

plt.show()
