import sys
import argparse
import subprocess as sp
import seppy

# Get command line arguments
parser = argparse.ArgumentParser(description="SigNoi Movie 1D")
requiredNamed = parser.add_argument_group('required named arguments')
requiredNamed.add_argument("-sigsc",help="Single Signal file",required=True)
requiredNamed.add_argument("-nozsc",help="Single Noise file",required=True)
requiredNamed.add_argument("-sigmc",help="Multi Signal file",required=True)
requiredNamed.add_argument("-nozmc",help="Multi Noise file",required=True)
requiredNamed.add_argument("-clip",help="Clip value",required=True)
requiredNamed.add_argument("-niter",help="Number of iterations to show movie",dest="niter")
parser.add_argument("-erase",action="store",dest="erase")
if(len(sys.argv) == 1):
  parser.print_help()
  sys.exit(1)

args = parser.parse_args()

sscin = args.sigsc
nscin = args.nozsc
smcin = args.sigmc
nmcin = args.nozmc
niter = args.niter
clip  = float(args.clip)

sepio = seppy.sepio()

scsaxes = sepio.read_header(sscin)
scnaxes = sepio.read_header(nscin)
mcsaxes = sepio.read_header(smcin)
mcnaxes = sepio.read_header(nmcin)

assert(scsaxes.n[1] == scnaxes.n[1]), "Single component Signal and noise \
    must have same number of iterations"

assert(mcsaxes.n[1] == mcnaxes.n[1]), "Multi component Signal and noise \
    must have same number of iterations"

scniter = scsaxes.n[1]
mcniter = mcsaxes.n[1]

if(args.niter == None):
  niter = max(scniter,mcniter)
else:
  niter = int(args.niter)
  assert(niter <= scniter or niter <= mcniter), "niter must be less sciter or mciter"

# Once I have reached the final iteration, of one 
# just keep plotting the last iteration

sciter = 0; mciter = 0
for iiter in range(niter):
  # Single component signal and noise
  if(iiter >= scniter):
    sciter = scniter-1
  else:
    sciter = iiter
  sigsc = "Window n2=1 f2=%d < %s | Wiggle yreverse=y \
      transp=y screenwd=4.0 zplot=0.2 poly=n crowd1=0.6 \
      label1='Time (s)' clip=%f wantaxis2=n title='SC Signal' \
      titlefat=2 labelfat=2 labelrot=n plotfat=2 wantframe=y \
      out=./Fig/sigsc-%d.v > /dev/null"%(sciter,sscin,clip,iiter)
  print sigsc
  print " "
  sp.check_call(sigsc,shell=True)
  nozsc = "Window n2=1 f2=%d < %s | Wiggle yreverse=y \
      transp=y screenwd=4.0 zplot=0.2 poly=n crowd1=0.6  \
      label1='Time (s)' clip=%f wantaxis2=n title='SC Noise' \
      titlefat=2 labelfat=2 labelrot=n plotfat=2 wantframe=y \
      wantaxis1=n out=./Fig/nozsc-%d.v > /dev/null"%(sciter,nscin,clip,iiter)
  sp.check_call(nozsc,shell=True)
  print nozsc
  print " "
  shftnozsc = "vppen xshift=3 < ./Fig/nozsc-%d.v > ./Fig/nozscshift-%d.v"%(iiter,iiter)
  sp.check_call(shftnozsc,shell=True)

  # Multicomponent signal and noise
  if(iiter >= mcniter):
    mciter = mcniter - 1
  else:
    mciter = iiter
  sigmc = "Window n2=1 f2=%d < %s | Wiggle yreverse=y \
      transp=y screenwd=4.0 zplot=0.2 poly=n crowd1=0.6 \
      label1='Time (s)' clip=%f wantaxis2=n title='MC Signal' \
      titlefat=2 labelfat=2 labelrot=n plotfat=2 wantframe=y \
      wantaxis1=n out=./Fig/sigmc-%d.v > /dev/null"%(mciter,smcin,clip,iiter)
  print sigmc
  print " "
  sp.check_call(sigmc,shell=True)
  shiftsigmc = "vppen xshift=6 < ./Fig/sigmc-%d.v > ./Fig/sigmcshift-%d.v"%(iiter,iiter)
  sp.check_call(shiftsigmc,shell=True)
  nozmc = "Window n2=1 f2=%d < %s | Wiggle yreverse=y \
      transp=y screenwd=4.0 zplot=0.2 poly=n crowd1=0.6 \
      label1='Time (s)' clip=%f wantaxis2=n title='%d' \
      titlefat=2 labelfat=2 labelrot=n plotfat=2 wantframe=y \
      wantaxis1=n out=./Fig/nozmc-%d.v > /dev/null"%(mciter,nmcin,clip,iiter,iiter)
  sp.check_call(nozmc,shell=True)
  print nozmc
  print " "
  shftnozmc = "vppen xshift=9 < ./Fig/nozmc-%d.v > ./Fig/nozmcshift-%d.v"%(iiter,iiter)
  sp.check_call(shftnozmc,shell=True)

  # Combine all figures
  comb = "vppen erase=n ./Fig/sigsc-%d.v ./Fig/nozscshift-%d.v \
      ./Fig/sigmcshift-%d.v ./Fig/nozmcshift-%d.v > \
      ./Fig/signoi-%d.v"%(iiter,iiter,iiter,iiter,iiter)
  print comb
  print " "
  sp.check_call(comb,shell=True)
  ps = "pstexpen ./Fig/signoi-%d.v ./Fig/signoi-%d.ps color=Y fat=1 \
      fatmult=1.5 invras=Y force="%(iiter,iiter)
  sp.check_call(ps,shell=True)
  pdf = "epstopdf  ./Fig/signoi-%d.ps"%(iiter)
  sp.check_call(pdf,shell=True)

# Clean up files
if(args.erase != None):
  rm = "rm -f ./Fig/sig*.v ./Fig/noz*.v rm ./Fig/signoi-*.v*"
  sp.check_call(rm,shell=True)

