/**
 * A program to blend multicomponent
 * acoustic shot gathers. Assumes
 * input shots are shot at a regular
 * time interval. Output shots are blended
 * at a random time interval.
 * @author: Joseph Jennings
 * @version: 2017.04.06
 */

#include "sep_params.h"
#include "../main/blend.h"
#include "../main/simpleplot.h"

int main(int argc, char **argv) {

  /* Input parameters */
  sep_params pars = sep_params(argc, argv);
  int adj = pars.get_int("adj",0);
  float maxdly = pars.get_float("maxdly",0.5);
  float mindly = pars.get_float("mindly",-0.5);
  std::vector<float> stimes = pars.get_vfloats("stimes",100);
  std::string enctag = pars.get_string("encshots","");

  sregf fin = sregf("in");

  if(fin.get_axis(4).n != 3)
    seperr("Error: last axis of data must be the component axis,\n");

  int nt = fin.get_axis(1).n;
  int nr = fin.get_axis(2).n;
  int ns = fin.get_axis(3).n;

  hypercube_float *mcdat = nullptr; hypercube_float *mcbdat = nullptr;
  hypercube_float *ocdat = nullptr; hypercube_float *ocbdat = nullptr;
  hypercube_float *medat = nullptr; hypercube_float *oedat  = nullptr;

  /* Shot interval in seconds */
  float ds = fin.get_axis(1).n*fin.get_axis(1).d;
  blend *gam = nullptr;

  if(adj) {
  } else {
    /* Read in data */
    mcdat = new hypercube_float(fin.get_axes(),true);
    fin.read_all("in",mcdat);
    medat = new hypercube_float(mcdat->get_axes(),true);
    std::vector<axis> ocaxes,ocbaxes,mcbaxes;
    /* Temporary array for a single component */
    ocaxes.push_back(mcdat->get_axis(1));
    ocaxes.push_back(mcdat->get_axis(2));
    ocaxes.push_back(mcdat->get_axis(3));
    ocdat = new hypercube_float(ocaxes,true);
    oedat = new hypercube_float(ocaxes,true);
    /* Construct blending operator */
    if(!stimes.empty()) {
      if(enctag.empty()) {
        gam = new blend(ocaxes,ds,stimes);
      } else {
        gam = new blend(ocaxes,ds,stimes,true);
      }
    } else {
      if(enctag.empty()) {
        gam = new blend(ocaxes,ds,maxdly,mindly);
      } else {
        gam = new blend(ocaxes,ds,maxdly,mindly,true);
      }
    }
    int nbs = gam->get_nshots();
    /* Temporary blended single component */
    ocbaxes.push_back(mcdat->get_axis(1)); ocbaxes.push_back(mcdat->get_axis(2));
    ocbaxes.push_back(axis(nbs,0.0,1.0));
    ocbdat = new hypercube_float(ocbaxes,true);
    /* Blended all components */
    mcbaxes.push_back(mcdat->get_axis(1)); mcbaxes.push_back(mcdat->get_axis(2));
    mcbaxes.push_back(axis(nbs,0.0,1.0)); mcbaxes.push_back(fin.get_axis(4));
    mcbdat = new hypercube_float(mcbaxes,true);
    /* Blend each component separately */
    for(int ic = 0; ic < mcdat->get_axis(4).n; ++ic) {
      memcpy(ocdat->vals,&mcdat->vals[ic*ns*nr*nt],sizeof(float)*ns*nt*nr);
      gam->forward(false,ocdat,ocbdat);
      gam->get_encshots(oedat);
      memcpy(&mcbdat->vals[ic*nbs*nr*nt],ocbdat->vals,sizeof(float)*nbs*nt*nr);
      memcpy(&medat->vals[ic*ns*nr*nt],oedat->vals,sizeof(float)*ns*nt*nr);
    }
    /* Write out the blended data */
    sregf bdout = sregf(mcbdat->get_axes());
    bdout.write_description("out");
    bdout.write_all("out",mcbdat);
    /* Write out the encoded data */
    if(!enctag.empty()) {
      sregf encout = sregf(medat->get_axes());
      encout.write_description("encshots");
      encout.write_all("encshots",medat);
    }
  }

  delete mcdat; delete ocdat; delete mcbdat; delete ocbdat;
  delete gam;
}
