/**
 * Creates multi-channel time series
 * via a multi-channel convolution operator
 * @author: Joseph Jennings
 * @version: 2017.05.08
 */

#include "sep_params.h"
#include "../main/weight.h"
#include "../main/mctcai.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  float f1 = pars.get_float("f1",1.0);
  float f2 = pars.get_float("f1",-0.3);
  float f3 = pars.get_float("f1",0.2);
  float f4 = pars.get_float("f1",1.0);

  sregf mcin = sregf("in");

  std::vector<axis> chaxes;
  chaxes.push_back(mcin.get_axis(1));

  int n = mcin.get_axis(1).n;

  hypercube_float *in = new hypercube_float(mcin.get_axes(),true);
  hypercube_float *ot = new hypercube_float(mcin.get_axes(),true);
  hypercube_float *p  = new hypercube_float(chaxes,true);
  hypercube_float *s  = new hypercube_float(chaxes,true);
  hypercube_float *op = new hypercube_float(chaxes,true);
  hypercube_float *os = new hypercube_float(chaxes,true);
  if(!p->check_match(s)) {
    seperr("Input channels must have same axes.\n");
  }

  hypercube_float *w1 = new hypercube_float(chaxes,true);
  hypercube_float *w2 = new hypercube_float(chaxes,true);
  hypercube_float *w3 = new hypercube_float(chaxes,true);
  hypercube_float *w4 = new hypercube_float(chaxes,true);

  /* Create four weighting operators */
  w1->set_val(f1); w2->set_val(f2);
  w3->set_val(f3); w4->set_val(f4);
  weight *wop1 = new weight(w1);
  weight *wop2 = new weight(w2);
  weight *wop3 = new weight(w3);
  weight *wop4 = new weight(w4);

  /* Construct multi-channel convolution operator */
  std::vector<my_operator*> wops;
  wops.push_back(wop1); wops.push_back(wop2);
  wops.push_back(wop3); wops.push_back(wop4);

  mctcai mcc = mctcai(wops);

  mcin.read_all("in",in);
  memcpy(p->vals,&in->vals[0*n],sizeof(float)*n);
  memcpy(s->vals,&in->vals[1*n],sizeof(float)*n);

  /* Create input and output vectors */
  super_vector *mod = new super_vector(p,s);
  super_vector *dat = new super_vector(op,os);

  mcc.forward(false,mod,dat);

  /* Copy data to out array */
  memcpy(&ot->vals[0*n],((hypercube_float*)dat->vecs[0])->vals,sizeof(float)*n);
  memcpy(&ot->vals[1*n],((hypercube_float*)dat->vecs[1])->vals,sizeof(float)*n);

  /* Write out data */
  sregf out = sregf(ot->get_axes());
  out.write_description("out");
  out.write_all("out",ot);

  /* Free memory */
  delete w1; delete w2; delete w3; delete w4;
}
