/**
 * Creates interleaved impulse data (multichannel)
 * for test case one
 * @author: Joseph Jennings
 * @version: 2017.05.08
 */

#include "sep_params.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  int n = pars.get_int("n",10000);
  float o = pars.get_float("o",0.0);
  float d = pars.get_float("d",0.004);
  int space  = pars.get_int("space",40);
  int offset = pars.get_int("offset",20);

  std::vector<axis> axes;
  axes.push_back(axis(n,o,d));

  hypercube_float *ch1 = new hypercube_float(axes,true);
  hypercube_float *ch2 = new hypercube_float(axes,true);
  ch1->zero(); ch2->zero();

  for(int i = 0; i < n-20; ++i) {
    if(i%space == 0) {
      ch1->vals[i]    = 1.0;
      fprintf(stderr,"i=%d %d\n",i,i%space);
      ch2->vals[i+offset] = 1.0;
    }
  }

  std::vector<axis> oaxes;
  oaxes.push_back(axes[0]);oaxes.push_back(axis(2,0.0,1.0));
  hypercube_float *datout = new hypercube_float(oaxes,true);
  memcpy(&datout->vals[0*n],ch1->vals,sizeof(float)*n);
  memcpy(&datout->vals[1*n],ch2->vals,sizeof(float)*n);

  sregf out = sregf(datout->get_axes());
  out.write_description("out");
  out.write_all("out",datout);

}
