/**
 * Computes the deblending model residual
 * after signal and noise separation
 * @author: Joseph Jennings
 * @version: 2017.03.29
 */

#include "sep_params.h"
#include "../main/simpleplot.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  int nrmlz = pars.get_int("nrmlz",1);

  sregf modin = sregf("in");
  sregf smvin = sregf("mov");

  int nt = modin.get_axis(1).n;
  int nr = modin.get_axis(2).n;
  int niter = smvin.get_axis(3).n;
  std::vector<axis> nrmxs;
  nrmxs.push_back(axis(niter,0.0,1.0));

  hypercube_float *mod  = new hypercube_float(modin.get_axes(),true);
  hypercube_float *rsd  = new hypercube_float(modin.get_axes(),true);
  hypercube_float *tsig = new hypercube_float(modin.get_axes(),true);
  hypercube_float *sig  = new hypercube_float(smvin.get_axes(),true);
  hypercube_float *nrm  = new hypercube_float(nrmxs,true);

  modin.read_all("in",mod);
  smvin.read_all("mov",sig);

  for(int iter = 0; iter < niter; ++iter) {
    memcpy(tsig->vals,&sig->vals[(iter)*nt*nr],sizeof(float)*nt*nr);
    rsd->scale_add(1.0,tsig,-1.0,mod);
    nrm->vals[iter] = rsd->dot(rsd);
  }

  if(nrmlz) {
    nrm->scale((1.0/nrm->my_max()));
  }

  sregf nout = sregf(nrm->get_axes());
  int one = 1;
  nout.write_description("out");
  putch("n2","d",&one);
  putch("n3","d",&one);
  nout.write_all("out",nrm);

  delete mod; delete rsd; delete tsig;
  delete sig; delete nrm;
}
