/**
 * Test for linearly interpolating a sparse grid
 * of filter coefficients. The first step
 * of linearly-varying convolution
 * @author: Joseph Jennings
 * @version: 2017.12.21
 */

#include "sregf.h"
#include "../main/config_parser.h"
#include "../main/lint_fast.h"
#include "../main/error_handle.h"

int main(int argc, char **argv) {

  ConfigParser cfg = ConfigParser(argc, argv, true);
  bool adj = cfg.get_bool("cmd", "adj", false);
  int j    = cfg.get_int("cmd", "j");
  int verb = cfg.get_int("cmd", "verb", 0);
  std::vector<int> lag;
  int nlag = 0, nd = 0;
  try {
    lag = cfg.get_ints("cmd", "lag");
    nlag = (int)lag.size();
  } catch (const InvalidArg &ex) {
    if(verb) fprintf(stderr,"No lags provided, assuming linearly increasing.\n");
  }
  if(!adj) {
    try {
      nd = cfg.get_int("cmd", "nd");
    } catch (const InvalidArg &ex) {
      fprintf(stderr, "For the forward operator, you must specify 'nd' (output size).\n");
      return 1;
    }
  }

  sregf fin = sregf("in");

  hypercube_float *dat = nullptr, *mod = nullptr;

  if(!adj) {
    mod = new hypercube_float(fin.get_axes(), true);
    if(nlag != 0 && nlag != mod->get_axis(1).n) {
      fprintf(stderr,"Number of lags provided must be same size as provided filter length.\n");
      return 1;
    }
    if(nlag == 0) {
      nlag = mod->get_axis(1).n;
    }
    fin.read_all("in", mod);
    std::vector<axis> daxes;
    daxes.push_back(axis(nlag,0.0,1.0));
    daxes.push_back(axis(nd  ,0.0,1.0));
    dat = new hypercube_float(daxes,true);
    lint_fast lf = lint_fast(nd, j, nlag);
    if(verb) lf.print_blockinfo();
    lf.forward(false, mod, dat);
    sregf fout = sregf(dat->get_axes());
    fout.write_description("out");
    fout.write_all("out", dat);
  }
  else {
  }

  /* Clean up */
  delete mod; delete dat;

}
