/**
 * Applies the forward and adjoint
 * of the linearly-varying
 * convolution operator
 * @author: Joseph Jennings
 * @version: 2017.03.16
 */

#include "sep_params.h"
#include "lv_conv1df.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  int adj = pars.get_int("adj",0);
  int j = pars.get_int("j",25);

  sregf modin = sregf("in");
  sregf lagin = sregf("lag");
  sregf fltin;

  hypercube_float *mod   = new hypercube_float(modin.get_axes(),true);
  hypercube_float *dat   = new hypercube_float(modin.get_axes(),true);
  hypercube_float *laghf = new hypercube_float(lagin.get_axes(),true);
  hypercube_float *flt = nullptr;

  modin.read_all("in",mod);
  lagin.read_all("lag",laghf);

  std::vector<int> lag;
  for(int il = 0; il < lagin.get_axis(1).n; ++il) {
    lag.push_back(static_cast<int>(laghf->vals[il]));
  }

  if(adj) {
  } else {
    fltin = sregf("flt");
    flt = new hypercube_float(fltin.get_axes(),true);
    fltin.read_all("flt",flt);
    lv_conv1df lv(mod->get_axis(1).n,j,lag,mod);
    fprintf(stderr,"numfilt=%d\n",lv.get_numfilt()[1]);
    lv.print_blockinfo();
    lv.forward(false,flt,dat);
    sregf datout = sregf(dat->get_axes());
    datout.write_description("out");
    datout.write_all("out",dat);
  }
  delete mod; delete dat; delete laghf; delete flt;
}
