/**
 * Tests the lv_conv1dmmc forward operator
 * @author: Joseph Jennings
 * @version: 2018.02.11
 */

#include "sregf.h"
#include "hypercube_float.h"
#include "sep_params.h"
#include "../main/error_handle.h"
#include "../main/lv_conv1dmmc.h"

int main(int argc, char **argv) {

  /* Get command line parameters */
  sep_params pars = sep_params(argc,argv);
  int verb = pars.get_int("verb",1);
  int nt = pars.get_int("nd",1000);
  int nc = pars.get_int("nc",2);
  int spikeloc = pars.get_int("sploc",499);
  std::vector<int> nlags = pars.get_auxvints("nlag",2,"in");
  std::vector<int> lag1s = pars.get_auxvints("lag1",50,"in");
  std::vector<int> lag2s = pars.get_auxvints("lag2",50,"in");
  int j = pars.get_auxint("j","in");

  /* Read in PEF */
  sregf pefin = sregf("in");
  hypercube_float *pef = new hypercube_float(pefin.get_axes(),true);
  pefin.read_all("in", pef);

  std::vector<axis> spaxes;
  spaxes.push_back(axis(nt,0.0,1.0));
  spaxes.push_back(axis(nc,0.0,1.0));

  hypercube_float *spike = new hypercube_float(spaxes,true);
  spike->vals[spikeloc] = 1.0;
  hypercube_float *imp = new hypercube_float(spaxes,true);

  /* Build the convolution operator */
  lv_conv1dmmc *lv1mc = nullptr;
  if(!nlags.empty()) {
    lv1mc = new lv_conv1dmmc(nt,j,nlags,pef);
  } else if(!lag1s.empty() && !lag2s.empty()) {
    if(lag1s.size() != lag2s.size()) {
      seperr("Lag 1 and lag 2 are not the same size. Each lag point "
          "requires one coordinate (axis 1 and axis 2).\n");
    }
    std::vector<std::pair<int,int>> lags;
    for(int il = 0; il < (int)lag1s.size(); ++il) {
      lags.push_back(std::pair<int,int>(lag1s[il],lag2s[il]));
    }
    lv1mc = new lv_conv1dmmc(nt,j,lags,pef);
  }

  if(verb) {
    lv1mc->print_blockinfo();
    fprintf(stderr,"\n");
    lv1mc->print_laginfo();
  }

  lv1mc->forward(false,spike,imp);

  /* Write out impulse response */
  sregf impout = sregf(imp->get_axes());
  impout.write_description("out");
  impout.write_all("out", imp);

}
