/**
 * Performs linearly-varying convolution
 * on a 2D image.
 * @author: Joseph Jennings
 * @version: 2017.03.19
 */

#include "sep_params.h"
#include "hypercube_float.h"
#include "../main/lv_conv2df.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  int adj = pars.get_int("adj",0);
  int verb = pars.get_int("verb",0);
  int maxlag = 50; std::vector<int> empt;
  std::vector<int> lag1 = pars.get_vints("lag1",maxlag,empt);
  std::vector<int> lag2 = pars.get_vints("lag2",maxlag,empt);
  std::vector<int> nlag = pars.get_vints("nlag",2,empt);
  std::vector<int> j    = pars.get_vints("j",2,empt);
  if(lag1.empty() && lag2.empty() && nlag.empty()) {
    seperr("Please supply either lag coordinates or the filter box dimensions\n");
  }
  if(j.empty()) {
    seperr("Please supply filter coarse sampling (j's).\n");
  }

  sregf imgin = sregf("in");
  sregf coefin; sregf fimgin;
  if(adj) {
    fimgin = sregf("fimg");
  } else {
    coefin = sregf("flt");
  }

  hypercube_float *img = new hypercube_float(imgin.get_axes(),true);
  hypercube_float *fmg = new hypercube_float(imgin.get_axes(),true);
  std::vector<int> nd;
  nd.push_back(imgin.get_axis(1).n); nd.push_back(imgin.get_axis(2).n);
  imgin.read_all("in",img);

  /* Build LV convolution operator */
  lv_conv2df lv2(nd,j,nlag,img);
  std::vector<int> nf = lv2.get_numfilt();
  if(verb) {
    lv2.print_laginfo();
    lv2.print_blockinfo();
  }

  /* Allocate for filter */
  std::vector<axis> faxes;
  faxes.push_back(axis(nf[0],0.0,1.0));
  faxes.push_back(axis(nf[1],0.0,1.0));
  faxes.push_back(axis(nf[2],0.0,1.0));
  hypercube_float *flt = new hypercube_float(faxes,true);

  if(adj) {
    fimgin.read_all("fimg",fmg);
    lv2.set_zerolag(flt);
    lv2.adjoint(false,flt,fmg);
    sregf fltout = sregf(flt->get_axes());
    fltout.write_description("out");
    fltout.write_all("out",flt);
  } else {
    coefin.read_all("flt",flt);
    lv2.forward(false,flt,fmg);
    sregf fmgout = sregf(fmg->get_axes());
    fmgout.write_description("out");
    fmgout.write_all("out",fmg);
  }

  delete img; delete flt; delete fmg;
}
