/**
 * Performs linearly-varying convolution
 * on a 2D image where the filter is an
 * auxiliary input
 * @author: Joseph Jennings
 * @version: 2017.03.23
 */

#include "sep_params.h"
#include "hypercube_float.h"
#include "../main/lv_conv2dm.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  int adj = pars.get_int("adj",0);
  int verb = pars.get_int("verb",0);
  int maxlag = 50; std::vector<int> empt;
  std::vector<int> lag1 = pars.get_vints("lag1",maxlag);
  std::vector<int> lag2 = pars.get_vints("lag2",maxlag);
  std::vector<int> nlag = pars.get_vints("nlag",2);
  std::vector<int> j    = pars.get_vints("j",2);
  if(lag1.empty() && lag2.empty() && nlag.empty()) {
    seperr("Please supply either lag coordinates or the filter box dimensions\n");
  }
  if(j.empty()) {
    seperr("Please supply filter coarse sampling (j's).\n");
  }

  sregf imgin  = sregf("in");
  sregf coefin = sregf("flt");

  hypercube_float *img = new hypercube_float(imgin.get_axes(),true);
  hypercube_float *fmg = new hypercube_float(imgin.get_axes(),true);
  hypercube_float *flt = new hypercube_float(coefin.get_axes(),true);
  coefin.read_all("flt",flt);

  std::vector<int> nd;
  nd.push_back(imgin.get_axis(1).n); nd.push_back(imgin.get_axis(2).n);
  /* Build LV convolution operator */
  lv_conv2dm lv2(nd,j,nlag,flt);
  if(verb) {
    lv2.print_laginfo();
    lv2.print_blockinfo();
  }

  sregf fout = sregf(img->get_axes());
  fout.write_description("out");
  if(adj) {
    imgin.read_all("in",fmg);
    lv2.adjoint(false,img,fmg);
    fout.write_all("out",img);
  } else {
    imgin.read_all("in",img);
    lv2.forward(false,img,fmg);
    fout.write_all("out",fmg);
  }

  delete img; delete flt; delete fmg;
}
