/**
 * Performs linearly-varying convolution
 * on a 3D dataset where the filter
 * is the operator
 * @author: Joseph Jennings
 * @version: 2017.03.23
 */

#include "sep_params.h"
#include "../main/lv_conv3dm.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  int adj = pars.get_int("adj",0);
  int vrb = pars.get_int("verb",0);
  int maxlag = 100;
  std::vector<int> lag1 = pars.get_vints("lag1",maxlag);
  std::vector<int> lag2 = pars.get_vints("lag2",maxlag);
  std::vector<int> lag3 = pars.get_vints("lag3",maxlag);
  std::vector<int> nlag = pars.get_vints("nlag",3);
  std::vector<int> j    = pars.get_vints("j",3);
  if(lag1.empty() && lag2.empty() && lag3.empty()) {
    if(nlag.empty()) {
      seperr("Please supply either lag coordinates or the filter box dimensions\n");
    }
    if(nlag.size() != 3) {
      seperr("Please supply three nlag values or a list of lag coordinates\n");
    }
  }
  if(j.empty()) {
    seperr("Please supply filter coarse sampling (j's).\n");
  }
  if(j.size() != 3) {
    seperr("Please supply three j values\n");
  }

  sregf imgin = sregf("in");
  sregf coefin = sregf("flt");

  hypercube_float *img = new hypercube_float(imgin.get_axes(),true);
  hypercube_float *fmg = new hypercube_float(imgin.get_axes(),true);
  hypercube_float *flt = new hypercube_float(coefin.get_axes(),true);
  imgin.read_all("in",img);
  coefin.read_all("flt",flt);

  std::vector<int> nd;
  nd.push_back(imgin.get_axis(1).n); nd.push_back(imgin.get_axis(2).n);
  nd.push_back(imgin.get_axis(3).n);
  /* Build the convolution operator */
  lv_conv3dm *lv3 = nullptr;
  if(!nlag.empty()) {
    lv3 = new lv_conv3dm(nd,j,nlag,flt);
  } else if(!lag1.empty() && !lag2.empty() && !lag3.empty()) {
    if(lag1.size() != lag2.size() || lag3.size() != lag2.size()) {
      seperr("Lag 1, lag 2 and lag 3 are not the same size. Each lag point "
          "requires one coordinate (axis 1, axis 2 and axis 3).\n");
    }
    std::vector<triple> lags;
    for(int il = 0; il < (int)lag1.size(); ++il) {
      lags.push_back(triple(lag1[il],lag2[il],lag3[il]));
    }
    lv3 = new lv_conv3dm(nd,j,lags,flt);
  }
  if(vrb) {
    lv3->print_blockinfo();
    fprintf(stderr,"\n");
    lv3->print_laginfo();
  }

  sregf fout = sregf(img->get_axes());
  fout.write_description("out");

  if(adj) {
    lv3->adjoint(false,img,fmg);
    fout.write_all("out",img);
  } else {
    lv3->forward(false,img,fmg);
    fout.write_all("out",fmg);
  }

  delete img; delete flt; delete fmg;
}
