/**
 * Makes a series of exponential functions
 * for a scalar TV PEF prediction
 * @author: Joseph Jennings
 * @version: 2017.05.25
 */

#include "sep_params.h"
#include "../main/wavelet.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  int nt = pars.get_int("nt",10000);
  float dt = pars.get_float("dt",0.004);
  float ot = pars.get_float("ot",0.0);
  std::vector<float> dcys = pars.get_vfloats("dcys",20);
  std::vector<float> dlys = pars.get_vfloats("dlys",20);

  if(dcys.size() != dlys.size()) { seperr("Dlys and Dcys must be same length.\n"); }

  std::vector<axis> taxes; taxes.push_back(axis(nt,ot,dt));

  hypercube_float *one = new hypercube_float(taxes,true);
  hypercube_float *tot = new hypercube_float(taxes,true);

  /* Loop over all provided decays and add */
  for(int id = 0; id < (int)dcys.size(); ++id) {
    wavelet wav = wavelet(taxes[0],dlys[id]);
    wav.expdecay(1.0,dcys[id],one);
    tot->scale_add(1.0,one,1.0);
  }

  /* Write out total wavelet */
  sregf fout = sregf(taxes);
  fout.write_description("out");
  fout.write_all("out",tot);
}
