/**
 * Extends the edges of a
 * velocity model for absorbing boundary
 * wave propagation
 * @author: Joseph Jennings
 * @version: 2017.01.28
 */

#include "sep_params.h"
#include "../main/padvel.h"
#include "../main/simpleplot.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);

  int adj  = pars.get_int("adj",0);
  int extx = pars.get_int("extx",0);
  int extz = pars.get_int("extz",0);
  int xloc = pars.get_int("xloc",0);
  int zloc = pars.get_int("zloc",0);
  int ord  = pars.get_int("ord",10);
  std::string mode = pars.get_string("mode","");

  sregf fin = sregf("in");
  hypercube_float *dat = nullptr;
  hypercube_float *pad = nullptr;
  padvel pv = padvel(zloc,xloc,ord);
  if(adj) {
    /* Create pad */
    pad = new hypercube_float(fin.get_axes(), true);
    fin.read_all("in",pad);
    std::vector<axis> dax;
    /* Create dat */
    dax.push_back(axis(fin.get_axis(1).n - extz - ord, fin.get_axis(1).o, fin.get_axis(1).d));
    dax.push_back(axis(fin.get_axis(2).n - extx - ord, fin.get_axis(2).o, fin.get_axis(2).d));
    dat = new hypercube_float(dax, true);
    pv.adjoint(false,dat,pad);
    sregf fout = sregf(dat->get_axes());
    fout.write_description("out");
    fout.write_all("out",dat);
  } else {
    /* Create dat */
    dat = new hypercube_float(fin.get_axes(), true);
    if(!mode.compare("rand")) {
      dat->random();
      sregf rout = sregf(dat->get_axes());
      rout.write_description("randout");
      rout.write_all("randout",dat);
    } else {
      fin.read_all("in",dat);
    }
    std::vector<axis> pax;
    pax.push_back(axis(fin.get_axis(1).n + extz + ord, fin.get_axis(1).o, fin.get_axis(1).d));
    pax.push_back(axis(fin.get_axis(2).n + extx + ord, fin.get_axis(2).o, fin.get_axis(2).d));
    pad = new hypercube_float(pax, true);
    pv.forward(false,dat,pad);
    //simpleplot::plot2d(pad);
    sregf fout = sregf(pad->get_axes());
    fout.write_description("out");
    fout.write_all("out",pad);
  }

  delete dat; delete pad;
}
