/**
 * Gets the inverse of the spectra of the PEF
 * @author: Joseph Jennings
 * @version: 2018.04.13
 */


#include <random>
#include "sep_params.h"
#include "hypercube_float.h"
#include "identity.h"
#include "../main/lv_conv2dm.h"
#include "../main/conj_grad.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  fprintf(stderr,"here\n");
  int adj = pars.get_int("adj",0);
  int verb = pars.get_int("verb",0);
  int niter = pars.get_int("niter",100);
  float eps = pars.get_float("eps",0.01);
  std::string rmov = pars.get_string("rmov","");
  std::string mmov = pars.get_string("mmov","");
  std::string gmov = pars.get_string("gmov","");
  int maxlag = 50; std::vector<int> empt;
  std::vector<int> lag1 = pars.get_vints("lag1",maxlag);
  std::vector<int> lag2 = pars.get_vints("lag2",maxlag);
  std::vector<int> nlag = pars.get_vints("nlag",2);
  std::vector<int> j    = pars.get_vints("j",2);
  if(lag1.empty() && lag2.empty() && nlag.empty()) {
    seperr("Please supply either lag coordinates or the filter box dimensions\n");
  }
  if(j.empty()) {
    seperr("Please supply filter coarse sampling (j's).\n");
  }

  sregf imgin  = sregf("in");
  sregf coefin = sregf("flt");

  hypercube_float *spec = new hypercube_float(imgin.get_axes(),true);
  hypercube_float *rand = new hypercube_float(imgin.get_axes(),true);
  hypercube_float *zro  = new hypercube_float(imgin.get_axes(),true);
  hypercube_float *flt = new hypercube_float(coefin.get_axes(),true);
  coefin.read_all("flt",flt);
  spec->zero();

  /* Create zero mean random numbers */
  rand->random();
  zro->set_val(rand->sum()/rand->get_n123());
  rand->scale_add(1.0, zro, -1.0);
  zro->zero();

  std::vector<int> nd;
  nd.push_back(imgin.get_axis(1).n); nd.push_back(imgin.get_axis(2).n);
  /* Build LV convolution operator */
  lv_conv2dm *lv2 = new lv_conv2dm(nd,j,nlag,flt);
  if(verb) {
    lv2->print_laginfo();
    lv2->print_blockinfo();
  }

  /* Create the identity operator */
  identity *iop = new identity();
  iop->set_domain(flt); iop->set_range(flt);

  /* Run the regularized inversion */
  conj_grad cg = conj_grad(niter);
  if(!rmov.empty()) {
    cg.set_resout("rmov");
  } if(!gmov.empty()) {
    cg.set_grdout("gmov");
  } if(!mmov.empty()) {
    cg.set_modout("mmov");
  }
  cg.run(lv2, iop, rand, zro, spec, eps);
  sregf spout = sregf(spec->get_axes());
  spout.write_description("out");
  spout.write_all("out", spec);

  /* Clean up */
  delete spec; delete flt; delete rand; delete zro;
}
