/**
 * Applies the forward or adjoint
 * of the polynomial division
 * operator with a user-provided filter
 * to a single time series
 * @author: Joseph Jennings
 * @version: 2017.03.16
 */

#include "sep_params.h"
#include "../main/polydiv.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);
  int adj = pars.get_int("adj",0);
  int rand = pars.get_int("rand",1);
  std::string randout = pars.get_string("randout","");
  int n = pars.get_int("nt",100);
  float d = pars.get_float("dt",1.0);
  float o = pars.get_float("ot",0.0);

  std::vector<axis> daxes;
  sregf datin;
  if(rand) {
    daxes.push_back(axis(n,o,d));
  } else {
    datin = sregf("in");
    daxes = datin.get_axes();
  }

  sregf fltin = sregf("flt");

  hypercube_float *dat = new hypercube_float(daxes,true);
  hypercube_float *div = new hypercube_float(daxes,true);
  hypercube_float *flt = new hypercube_float(fltin.get_axes(),true);

  fltin.read_all("flt",flt);

  polydiv dec(flt);

  sregf dout = sregf(dat->get_axes());
  dout.write_description("out");

  if(adj) {
    if(!rand) {
      datin.read_all("in",div);
    } else {
      div->random();
    }
    dec.adjoint(false,dat,div);
    dout.write_all("out",dat);
    if(!randout.empty()) {
      sregf wrand = sregf(div->get_axes());
      wrand.write_description("randout");
      wrand.write_all("randout",div);
    }
  } else {
    if(!rand) {
      datin.read_all("in",dat);
    } else {
      dat->random();
    }
    dec.forward(false,dat,div);
    dout.write_all("out",div);
    if(!randout.empty()) {
      sregf wrand = sregf(dat->get_axes());
      wrand.write_description("randout");
      wrand.write_all("randout",dat);
    }
  }

  delete dat; delete flt; delete div;
}
