/**
 * Estimates a scalar time-variant
 * PEF on a gather
 * @author: Joseph Jennings
 * @version: 2017.06.21
 */

#include "sep_params.h"
#include "../main/sctvpef.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc,argv);
  int verb     = pars.get_int("verb",0);
  int gap      = pars.get_int("gap",1);
  int na       = pars.get_int("na",10);
  int lambda   = pars.get_int("lambda",200);

  sregf datin = sregf("in");
  int nt  = datin.get_axis(1).n;
  int ntr = datin.get_axis(2).n;
  std::vector<axis> at; at.push_back(datin.get_axis(1));

  std::vector<axis> faxes;
  faxes.push_back(axis(na,0.0,1.0));

  hypercube_float *dat = new hypercube_float(datin.get_axes(),true);
  hypercube_float *rsd = new hypercube_float(datin.get_axes(),true);
  hypercube_float *tr  = new hypercube_float(at,true);
  hypercube_float *e   = new hypercube_float(at,true);

  /* Initialize values */
  std::vector<float> A(na,0.0); e->zero();

  datin.read_all("in",dat);

  /* Loop over all traces */
  for(int itr = 0; itr < ntr; ++itr) {
    e->zero(); tr->zero();
    /* Build the PEF finder */
    sctvpef pef = sctvpef(A, gap, lambda, verb);
    /* Get the current trace */
    memcpy(tr->vals,&dat->vals[itr*nt],sizeof(float)*nt);
    A = pef.findpef(tr,e);
    memcpy(&rsd->vals[itr*nt],e->vals,sizeof(float)*nt);
  }

  /* Write residual */
  sregf rout = sregf(rsd->get_axes());
  rout.write_description("out");
  rout.write_all("out",rsd);

  delete dat; delete rsd;
}
