/**
 * Test the adjoint of the 1D linearly-varying
 * convolution operator. Adjoint returns the
 * filter.
 * @author: Joseph Jennings
 * @version: 2017.12.20
 */

#include <limits.h>
#include <random>
#include "gtest/gtest.h"
#include "axis.h"
#include "hypercube_float.h"
#include "../main/lv_conv1df.h"
#include "../main/simpleplot.h"

class lv_conv1dftest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(lv_conv1dftest,dptest) {

  int nd = 100, j = 25, nlag = 5;

  std::vector<axis> daxes,faxes;
  daxes.push_back(axis(nd,0.0,1.0));
  hypercube_float *aux = new hypercube_float(daxes,true);
  aux->random();

  lv_conv1df lv1 = lv_conv1df(nd,j,nlag,aux);

  std::vector<int> nf = lv1.get_numfilt();
  faxes.push_back(axis(nf[0],0.0,1.0));
  faxes.push_back(axis(nf[1],0.0,1.0));

  hypercube_float *m  = new hypercube_float(faxes,true);
  hypercube_float *ms = new hypercube_float(faxes,true);
  hypercube_float *d  = new hypercube_float(daxes,true);
  hypercube_float *ds = new hypercube_float(daxes,true);

  m->random(); d->random();

  /* Add == false */
  lv1.forward(false,m,ds);
  double dotd = d->dot(ds);
  lv1.adjoint(false,ms,d);
  double dotm = m->dot(ms);
  fprintf(stderr,"dotd=%f dotm=%f\n",dotd,dotm);
  ASSERT_NEAR(dotd,dotm,0.000001);

  /* Add == true */
  lv1.forward(true,m,ds);
  double adotd = d->dot(ds);
  lv1.adjoint(true,ms,d);
  double adotm = m->dot(ms);
  fprintf(stderr,"adotd=%f adotm=%f\n",adotd,adotm);
  ASSERT_NEAR(adotd,adotm,0.00001);

  delete m; delete ms; delete d; delete ds;
}
