/**
 * Test the adjoint of the linearly-varying
 * 1D multicomponent convolution operator
 * @author: Joseph Jennings
 * @version: 2018.02.10
 */

#include <limits.h>
#include <random>
#include "gtest/gtest.h"
#include "../main/lv_conv1dfmc.h"
#include "../main/simpleplot.h"
#include "axis.h"
#include "hypercube_float.h"

class lv_conv1dfmctest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(lv_conv1dfmctest,dptest) {

  std::vector<int> nd;
  nd.push_back(10); nd.push_back(2);
  int j = 3;

  std::vector<std::pair<int,int>> lag;
  lag.push_back(std::pair<int,int>(0,0));
  lag.push_back(std::pair<int,int>(1,0));
  lag.push_back(std::pair<int,int>(2,0));
  lag.push_back(std::pair<int,int>(2,1));

  std::vector<axis> daxes,faxes;
  daxes.push_back(axis(nd[0],0.0,1.0));
  daxes.push_back(axis(nd[1],0.0,1.0));
  hypercube_float *aux = new hypercube_float(daxes,true);
  aux->random();

  lv_conv1dfmc lv1mc = lv_conv1dfmc(nd[0],j,lag,aux);

  std::vector<int> nf = lv1mc.get_numfilt();
  faxes.push_back(axis(nf[0],0.0,1.0));
  faxes.push_back(axis(nf[1],0.0,1.0));

  hypercube_float *m  = new hypercube_float(faxes,true);
  hypercube_float *ms = new hypercube_float(faxes,true);
  hypercube_float *d  = new hypercube_float(daxes,true);
  hypercube_float *ds = new hypercube_float(daxes,true);

  m->random(); d->random();

  /* Add == false */
  lv1mc.forward(false,m,ds);
  double dotd = d->dot(ds);
  lv1mc.adjoint(false,ms,d);
  double dotm = m->dot(ms);
  fprintf(stderr,"dotd=%f dotm=%f\n",dotd,dotm);
  ASSERT_NEAR(dotd,dotm,0.000001);

  /* Add == true */
  lv1mc.forward(true,m,ds);
  double adotd = d->dot(ds);
  lv1mc.adjoint(true,ms,d);
  double adotm = m->dot(ms);
  fprintf(stderr,"adotd=%f adotm=%f\n",adotd,adotm);
  ASSERT_NEAR(adotd,adotm,0.00001);

  delete m; delete ms; delete d; delete ds;
}
