/**
 * Tests the adjoint of the masked
 * convolution operator used in 1D, linearly-varying
 * non-stationary, PEF esimation
 * @author: Joseph Jennings
 * @version: 2017.12.21
 */

#include <limits.h>
#include <random>
#include "gtest/gtest.h"
#include "../main/lv_conv1df.h"
#include "../main/lv_conv1dmask.h"
#include "../main/simpleplot.h"
#include "axis.h"
#include "hypercube_float.h"
#include "combo_oper.h"

class lv_conv1dmasktest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(lv_conv1dmasktest,dptest) {

  int nd = 100, j = 25, nlag = 5;

  std::vector<axis> daxes,faxes;
  daxes.push_back(axis(nd,0.0,1.0));
  hypercube_float *aux = new hypercube_float(daxes,true);
  aux->random();

  lv_conv1df *lv1  = new lv_conv1df(nd,j,nlag,aux);
  lv_conv1dmask *k = new lv_conv1dmask();

  std::vector<int> nf = lv1->get_numfilt();
  faxes.push_back(axis(nf[0],0.0,1.0));
  faxes.push_back(axis(nf[1],0.0,1.0));

  hypercube_float *m  = new hypercube_float(faxes,true);
  hypercube_float *ms = new hypercube_float(faxes,true);
  hypercube_float *d  = new hypercube_float(daxes,true);
  hypercube_float *ds = new hypercube_float(daxes,true);

  lv1->set_domain(m); lv1->set_range(d);
  k->set_domain(m); k->set_range(m);
  chain_op dk = chain_op(k,lv1);

  m->random(); d->random();

  /* Add == false */
  dk.forward(false,m,ds);
  double dotd = d->dot(ds);
  dk.adjoint(false,ms,d);
  double dotm = m->dot(ms);
  fprintf(stderr,"dotd=%f dotm=%f\n",dotd,dotm);
  ASSERT_NEAR(dotd,dotm,0.000001);

  delete m; delete ms; delete d; delete ds;
}
