/**
 * Test the adjoint of the linearly-varying
 * convolution operator
 * @author: Joseph Jennings
 * @version: 2017.03.19
 */

#include <limits.h>
#include <random>
#include "gtest/gtest.h"
#include "../main/lv_conv2df.h"
#include "../main/simpleplot.h"
#include "axis.h"
#include "hypercube_float.h"

class lv_conv2dftest: public ::testing::Test {
  protected:
    virtual void SetUp(){
    }

    virtual void TearDown() {
    }
};

TEST_F(lv_conv2dftest,dptest) {

  std::vector<int> nd, j;
  nd.push_back(10); nd.push_back(3);
  j.push_back(3); j.push_back(3);

  std::vector<std::pair<int,int>> lag;
  lag.push_back(std::pair<int,int>(0,0));
  lag.push_back(std::pair<int,int>(1,0));
  lag.push_back(std::pair<int,int>(2,0));
  lag.push_back(std::pair<int,int>(2,1));

  std::vector<axis> daxes,faxes;
  daxes.push_back(axis(nd[0],0.0,1.0));
  daxes.push_back(axis(nd[1],0.0,1.0));
  hypercube_float *aux = new hypercube_float(daxes,true);
  aux->random();

  lv_conv2df lv2 = lv_conv2df(nd,j,lag,aux);

  lv2.print_blockinfo();
  fprintf(stderr,"\n");
  lv2.print_laginfo();

  std::vector<int> nf = lv2.get_numfilt();
  faxes.push_back(axis(nf[0],0.0,1.0));
  faxes.push_back(axis(nf[1],0.0,1.0));
  faxes.push_back(axis(nf[2],0.0,1.0));

  hypercube_float *m  = new hypercube_float(faxes,true);
  hypercube_float *ms = new hypercube_float(faxes,true);
  hypercube_float *d  = new hypercube_float(daxes,true);
  hypercube_float *ds = new hypercube_float(daxes,true);

  m->random(); d->random();

  /* Add == false */
  lv2.forward(false,m,ds);
  double dotd = d->dot(ds);
  lv2.adjoint(false,ms,d);
  double dotm = m->dot(ms);
  fprintf(stderr,"dotd=%f dotm=%f\n",dotd,dotm);
  ASSERT_NEAR(dotd,dotm,0.000001);

  /* Add == true */
  lv2.forward(true,m,ds);
  double adotd = d->dot(ds);
  lv2.adjoint(true,ms,d);
  double adotm = m->dot(ms);
  fprintf(stderr,"adotd=%f adotm=%f\n",adotd,adotm);
  ASSERT_NEAR(adotd,adotm,0.00001);

  delete m; delete ms; delete d; delete ds;
}
