/**
 * Tests the findpef method within
 * the scalar TV PEF class
 * @author: Joseph Jennings
 * @version: 2017.06.18
 */

#include "sep_params.h"
#include "../main/sctvpef.h"

int main(int argc, char **argv) {

  sep_params pars = sep_params(argc, argv);
  int verb     = pars.get_int("verb",0);
  int gap      = pars.get_int("gap",1);
  int na       = pars.get_int("na",10);
  int lambda   = pars.get_int("lambda",200);
  std::string flttag = pars.get_string("fflt");

  sregf in = sregf("in");

  std::vector<axis> faxes;
  faxes.push_back(axis(na,0.0,1.0));

  hypercube_float *y   = new hypercube_float(in.get_axes(),true);
  hypercube_float *A   = new hypercube_float(faxes,true);
  hypercube_float *e   = new hypercube_float(in.get_axes(),true);
  A->zero(); e->zero();

  /* Read in y */
  in.read_all("in",y);

  sctvpef pef = sctvpef(na, gap, lambda, verb);
  A->vals = (pef.findpef(y,e)).data();

  /* Output error */
  sregf eout = sregf(e->get_axes());
  eout.write_description("out");
  eout.write_all("out",e);

  /* Output final filter */
  if(!flttag.empty()) {
    sregf aout = sregf(A->get_axes());
    aout.write_description("fflt");
    aout.write_all("fflt",A);
  }

  /* Clean up */
  delete y; delete A; delete e;

}
